#include "categoryitem.h"
#include <qlistview.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qtextstream.h>
#include <iostream>
#include "resources/checked.xpm"
#include "resources/unchecked.xpm"

CCategoryItem::CCategoryItem(QListViewItem *parent, const QString &text,
			     int idTop,  int idNode,  int idParent, bool checkable)
  : CGenericItem(parent, text), nTopLevel(idTop), nNodeID(idNode), nParentID(idParent),
    bCheckable(checkable)
{  
  setChecked(false);
}

CCategoryItem::CCategoryItem(QListView *parent, const QString &text,
			     int idTop, int idNode,  int idParent, bool checkable)
  : CGenericItem(parent, text), nTopLevel(idTop), nNodeID(idNode), nParentID(idParent),
    bCheckable(checkable)
{
  setChecked(false);
}

void CCategoryItem::setRecursiveChecked(bool set)
{

  CCategoryItem *sub;
  for(sub = (CCategoryItem *)firstChild(); sub; sub = (CCategoryItem *)sub->nextSibling()) {
    sub->setChecked(set);
    sub->setRecursiveChecked(set);
  }
  setChecked(set);

}
void CCategoryItem::setChecked(bool set)
{
  if(!bCheckable) 
    return;

  if(set) 
    setPixmap(0, QPixmap(checked_xpm));
  else
    setPixmap(0, QPixmap(unchecked_xpm));

  bChecked = set;
}

void CCategoryItem::activate()
{

  if(!bCheckable)
    return;

  setRecursiveChecked(!bChecked);
  if(bChecked == true)
    return;

  // If unselecting, also unselect every parent category
  CCategoryItem *p = (CCategoryItem *)parent();

  while(p) {
    p->setChecked(false);
    p = (CCategoryItem *)p->parent();
  }

 
  
}


bool CCategoryItem::anyChildrenSelected()
{
  CCategoryItem *sub;
  for(sub = (CCategoryItem *)firstChild(); sub; sub = (CCategoryItem *)sub->nextSibling()) {
    if(sub->getChecked())
      return true;
    if(sub->anyChildrenSelected()) // recursive
      return true;
  }
  return false;
}

CCategoryItem *CCategoryItem::getItemForID(int nID)
{
  CCategoryItem *ret, *item = (CCategoryItem *)firstChild();

  while(item) {
    if(item->getNodeID() == nID)
      return (item);
    if( (ret = item->getItemForID(nID)) != 0 )
      return (ret);
    item = (CCategoryItem *)item->nextSibling();
  }
  return 0;
}

QString CCategoryItem::getQueryString()
{
  CCategoryItem *item = (CCategoryItem *)firstChild();
  QString strTmp, strQuery;

  while(item) {
    if(item->getChecked()) {
      strTmp.sprintf(",%d", item->getNodeID());
      strQuery += strTmp;
    }

    strQuery += item->getQueryString();
    item = (CCategoryItem *)item->nextSibling();
  }

  return strQuery;
}

void CCategoryItem::writeSelf(QTextStream &t)
{
  t << nTopLevel << "," << nParentID << "," << nNodeID << ",\"" << text(0) << "\"," << (bChecked?1:0) << endl;
}
bool CCategoryItem::saveChildren(QTextStream &t)
{
  CCategoryItem *item = (CCategoryItem *)firstChild();

  while(item) {
    item->writeSelf(t);
    item->saveChildren(t);
    item = (CCategoryItem *)item->nextSibling();
  }
  return true;
}
    








