
#include "pager.h"
#include <qapplication.h>
#include <qwindowsstyle.h>
#include <qpixmap.h>
#include <qbitmap.h>
// Global flags

bool bVulnTest = false;
bool bDebug = false;

CMain *gPagerMain;
int main(int argc, char **argv)
{
  QApplication a(argc, argv);
  QWindowsStyle *ws = new QWindowsStyle;

  a.setStyle(ws);

  gPagerMain = new CMain;
  a.setMainWidget(gPagerMain);
  for(int i = 0; i < a.argc(); i++) {
    if(strcmp(a.argv()[i], "-vulntest") == 0)
      bVulnTest = true;
    if(strcmp(a.argv()[i], "-debug") == 0)
      bDebug = true;
  }
  
  gPagerMain->setCaption("SecurityFocus Pager V3.0");
  gPagerMain->show();
  gPagerMain->update();

  return a.exec();
}


 CMain::CMain() 
   : QWidget(0, 0, WStyle_NormalBorder), bInit(false) {


   gPagerMain = this;
   thePager = new CPager(this, 0);

   update_button = new CPagerButton(this);
   config_button = new CPagerButton(this);
   search_button = new CPagerButton(this);
   min_button = new CPagerButton(this);
   exit_button = new CPagerButton(this);
   help_button = new CPagerButton(this);

   plain_buttons = new QHBox(this);
   plain_buttons->hide();
   config_plain = new QPushButton("Config", plain_buttons); 
   search_plain = new QPushButton("Search", plain_buttons); 
   update_plain = new QPushButton("Update", plain_buttons); 
   help_plain = new QPushButton("Help", plain_buttons);     

    QObject::connect(update_plain, SIGNAL(clicked()), thePager, SLOT(slotUpdateContent()));
    QObject::connect(config_plain, SIGNAL(clicked()), thePager, SLOT(slotDoConfig()));
    QObject::connect(search_plain, SIGNAL(clicked()), thePager, SLOT(slotDoSearch()));
    QObject::connect(help_plain, SIGNAL(clicked()), thePager, SLOT(doHelp()));

    QToolTip::add(update_button, "Manually request latest articles from SecurityFocus");
    QToolTip::add(config_button, "Configure the SecurityFocus Pager");
    QToolTip::add(search_button, "Search the SecurityFocus web site");
    QToolTip::add(min_button, "Minimize the pager");
    QToolTip::add(exit_button, "Close the pager");
    QToolTip::add(help_button, "Read the pager documentation in your web browser");

    QObject::connect(update_button, SIGNAL(clicked()), thePager, SLOT(slotUpdateContent()));
    QObject::connect(config_button, SIGNAL(clicked()), thePager, SLOT(slotDoConfig()));
    QObject::connect(search_button, SIGNAL(clicked()), thePager, SLOT(slotDoSearch()));
    QObject::connect(min_button, SIGNAL(clicked()), this, SLOT(showMinimized()));
    QObject::connect(exit_button, SIGNAL(clicked()), qApp, SLOT(quit()));
    QObject::connect(help_button, SIGNAL(clicked()), thePager, SLOT(doHelp()));

    QObject::connect(qApp, SIGNAL(aboutToQuit()), thePager, SLOT(quit()));
    setFocusPolicy(QWidget::ClickFocus);
    setIcon(QPixmap(icon1_xpm));
    ad = new CBanner(this);
    QToolTip::add(ad, "Double click to visit this sponsor");
    bInit = true;
    thePager->applySkin();

    QObject::connect(thePager, SIGNAL(newIcon(bool)), this, SLOT(slotChangeIcon(bool)));

  }


 bool CMain::eventFilter(QObject *object, QEvent *event) {
    if(event->type() == QEvent::MouseButtonPress) {
      QMouseEvent *mevent = (QMouseEvent *)event;
      QWidget *widget = (QWidget *)object;
      QMouseEvent *me = new QMouseEvent(mevent->type(), mevent->pos() + widget->pos(), mevent->button(), mevent->state());
      mousePressEvent((QMouseEvent *)me);
      delete me;
    }
    else if (event->type() == QEvent::MouseMove)
      mouseMoveEvent((QMouseEvent *)event);
    else if(event->type() == QEvent::MouseButtonRelease)
      mouseReleaseEvent((QMouseEvent *)event);
    else if(event->type() == QEvent::MouseButtonDblClick) {
      if(object == logo)
	thePager->SurfItUp("http://www.securityfocus.com", true);
    }
    return false;
  }


