#include "netinventory.h"
#include "vulncatitem.h"
#include "wwwserver.h"
#include "newentitydialog.h"
#include "customdialog.h"
#include "customitem.h"
#include "config.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qlistview.h>
#include <qcombobox.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qheader.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <iostream>
#include <stdlib.h>
#include <stdio.h>
extern QString strPagerDir;
extern bool bDebug;

CNetInventory::CNetInventory(CWWWServer *www, CConfig *pConfig, QWidget *parent, const char *name)
  : QWidget(parent, name), m_nVersion(0)
{
  m_pConfig = pConfig;
  wwwServer = www;
  strTimestamp = "0";
  setup();
}

void CNetInventory::setup()
{
  QFile f(strPagerDir + "netinventory.dat");
 

  QHBoxLayout *hlayout1 = new QHBoxLayout(this);
  QVBoxLayout *vlayout0 = new QVBoxLayout();
  QHBoxLayout *hlayout2 = new QHBoxLayout();
  QHBoxLayout *hlayout3 = new QHBoxLayout();
  QVBoxLayout *vlayout1 = new QVBoxLayout();
  QVBoxLayout *vlayout2 = new QVBoxLayout();
  //  QVBoxLayout *vlayout3 = new QVBoxLayout();
  
  m_pVulnTree = new QListView(this);
  m_pVulnTree->addColumn( QString("       "));
  m_pVulnTree->setRootIsDecorated(true);
  m_pVulnTree->header()->hide();
  m_pVulnTree->setMinimumWidth(200);
  QObject::connect(m_pVulnTree, SIGNAL(selectionChanged()), SLOT(slotVulnSelected()));

  QLabel *customLabel = new QLabel("Custom Products", this);
  customLabel->setAlignment(AlignCenter);
  QLabel *productLabel = new QLabel("Products", this);
  productLabel->setAlignment(AlignCenter);

  m_pNewCustomButton = new QPushButton("&Add", this);
  QObject::connect(m_pNewCustomButton, SIGNAL(clicked()), this, SLOT(slotNewCustom()));
  
  m_pDelCustomButton = new QPushButton("&Remove", this);
  m_pDelCustomButton->setEnabled(false);
  QObject::connect(m_pDelCustomButton, SIGNAL(clicked()), SLOT(slotDeleteCustom()));

  m_pEditCustomButton = new QPushButton("&Edit", this);
  m_pEditCustomButton->setEnabled(false);
  QObject::connect(m_pEditCustomButton, SIGNAL(clicked()), SLOT(slotEditCustom()));

  m_pCustomList = new QListView(this);
  m_pCustomList->addColumn( QString("Name"));
  m_pCustomList->addColumn( QString("Keywords"));
  m_pCustomList->setColumnWidth(0, 100);
  m_pCustomList->setColumnWidth(1, 133);
  m_pCustomList->setMinimumHeight(100);
  QObject::connect(m_pCustomList, SIGNAL(selectionChanged()), SLOT(slotCustomSelected()));

  m_pAddButton = new QPushButton("->", this);
  m_pAddButton->setEnabled(false);
  QObject::connect(m_pAddButton, SIGNAL(clicked()), this, SLOT(slotAddProduct()));
  
  m_pRemoveButton = new QPushButton("<-", this);
  m_pRemoveButton->setEnabled(false);
  QObject::connect(m_pRemoveButton, SIGNAL(clicked()), this, SLOT(slotRemoveProduct()));
  

  m_pClearButton  = new QPushButton("<<", this);
  m_pClearButton->setEnabled(false);
  QObject::connect(m_pClearButton, SIGNAL(clicked()), this, SLOT(slotClearProducts()));

  m_pEntityCombo = new QComboBox(this);
  m_pEntityCombo->insertItem("All (By Profile)");
  m_pEntityCombo->insertItem("All (Combined)");
  QObject::connect(m_pEntityCombo, SIGNAL(activated(int)), SLOT(slotChangeEntity(int)));

  m_pVulnList = new QListView(this);
  m_pVulnList->addColumn(QString("Vendor"));
  m_pVulnList->addColumn(QString("Product"));
  m_pVulnList->setMinimumWidth(200);
  m_pVulnList->setColumnWidth(0, 160);
  m_pVulnList->setColumnWidth(1, 150);
  QObject::connect(m_pVulnList, SIGNAL(selectionChanged()), SLOT(slotVulnListSelected()));

  QPushButton *newButton = new QPushButton("&New", this);
  QObject::connect(newButton, SIGNAL(clicked()), this, SLOT(slotNewEntity()));

  m_pDeleteProfile = new QPushButton("&Delete", this);
  m_pDeleteProfile->setEnabled(false);
  QObject::connect(m_pDeleteProfile, SIGNAL(clicked()), SLOT(slotDeleteEntity()));

  m_pRenameProfile = new QPushButton("R&ename", this);
  m_pRenameProfile->setEnabled(false);
  QObject::connect(m_pRenameProfile, SIGNAL(clicked()), SLOT(slotRenameEntity()));

  vlayout0->addSpacing(10);
  vlayout0->addWidget(productLabel);
  vlayout0->addWidget(m_pVulnTree);
  vlayout0->addSpacing(15);
  vlayout0->addWidget(customLabel);
  vlayout0->addWidget(m_pCustomList);
  vlayout0->addLayout(hlayout2);

  hlayout2->addWidget(m_pNewCustomButton, 3);
  hlayout2->addStretch(1);
  hlayout2->addWidget(m_pDelCustomButton, 3);
  hlayout2->addStretch(1);
  hlayout2->addWidget(m_pEditCustomButton, 3);

  hlayout3->addWidget(newButton,3);
  hlayout3->addStretch(1);
  hlayout3->addWidget(m_pDeleteProfile, 3);
  hlayout3->addStretch(1);
  hlayout3->addWidget(m_pRenameProfile, 3);

  hlayout1->addLayout(vlayout0, 1);
  hlayout1->addSpacing(10);
  hlayout1->addLayout(vlayout1, 1);
  hlayout1->addSpacing(10);
  hlayout1->addLayout(vlayout2, 5);

  vlayout1->addStretch(3);
  vlayout1->addWidget(m_pAddButton,0);
  vlayout1->addSpacing(50);
  vlayout1->addWidget(m_pRemoveButton,0);
  vlayout1->addSpacing(50);
  vlayout1->addWidget(m_pClearButton,0);
  vlayout1->addStretch(3);
  
  QLabel *l = new QLabel("Current Profile", this);
  l->setAlignment(AlignCenter);
  vlayout2->addSpacing(10);
  vlayout2->addWidget(l);
  vlayout2->addSpacing(10);
  vlayout2->addLayout(hlayout3);
  vlayout2->addSpacing(10);
  vlayout2->addWidget(m_pEntityCombo);
  vlayout2->addWidget(m_pVulnList);
  QObject::connect(wwwServer, SIGNAL(vulnCategoryUpdateDone()), this, SLOT(slotLoadVulnTree()));
  populateVulnTree();
  if(f.exists())
    slotLoad("netinventory.dat");

  m_pEntityCombo->setCurrentItem(0);  
  setAllByProfile();


}

CNetworkEntity * CNetInventory::addEntity(QString strName)
{
  CNetworkEntity *p;

  if(strName.isEmpty()) 
    return pCurrentEntity;

  m_pEntityCombo->insertItem(strName, -1);
  // Switch to newly added item
  // this will emit selection changed and update list view
  m_pEntityCombo->setCurrentItem(m_pEntityCombo->count() - 1);
  m_pVulnList->clear();

  m_Entities.append(p =  new CNetworkEntity(strName, m_pEntityCombo->count() - 1));
  return p;
}
void  
CNetInventory:: populateVulnTree()
{

  QFile *f = new QFile(strPagerDir + "vulncategory.dat");
  if(!f->exists()) {
    wwwServer->queueRequest(CWWWRequest::R_UPDATEVULN, NULL);
    delete f;
    return;
  }
  delete f;
  slotLoadVulnTree();
}

void CNetInventory::slotLoadVulnTree()
{
  int pid, id, n;
  QString strDesc, strTmp;

  QFile *f = new QFile(strPagerDir + "vulncategory.dat");
  
  if(!f->open(IO_ReadOnly)) {
    delete f;
    return;
  }
  m_pVulnTree->clear();
  while(!f->atEnd()) {
    f->readLine(strTmp, 1024);
    if(strTmp.find("VERSION=") == 0) {
      sscanf(strTmp, "VERSION=%d", &n);
      if(n > 0)
	m_nVersion = n;
    }
    if(sscanf(strTmp.ascii(), "%d,%d,", &pid, &id) != 2)
      continue;

    n = strTmp.find('\"');
    strDesc = strTmp.mid(n + 1);
    n = strDesc.findRev('\"');
    if(n != -1)
      strDesc.truncate(n);

    if(pid == 0) // no parent
      (void) new CVulnCatItem(m_pVulnTree, strDesc, id);
    else
      (void) new CVulnCatItem(id2VulnItem(pid, false), strDesc, id);
    
    
  }

  f->close();
  delete f;
}
CCustomItem *CNetInventory::id2CustomItem(int id)
{
  for(CCustomItem *item = (CCustomItem *)m_pCustomList->firstChild(); item; item = (CCustomItem *)item->nextSibling())
    if(item->getID() == id)
      return item;
  return NULL;
}
CVulnCatItem *CNetInventory::id2VulnItem(int id, bool product)
{
  CVulnCatItem *item = (CVulnCatItem *)m_pVulnTree->firstChild();
  CVulnCatItem *subitem;

  while(item) {
    if(item->getID() == id && (!product))
      return item;

    if(product) {
      subitem = (CVulnCatItem *)item->firstChild();
      while(subitem) {
	if(subitem->getID() == id)
	  return subitem;
	subitem = (CVulnCatItem *)subitem->nextSibling();
      }
    }
    
    item = (CVulnCatItem *)item->nextSibling();
  }
  return NULL;
}

bool CNetInventory::entityExists(QString strName)
{
  CNetworkEntity *item;

  for(item = m_Entities.first(); item; item = m_Entities.next()) 
    if(item->getName() == strName)
      return true;

  return false;
}

void CNetInventory::slotNewEntity()
{
  CNewEntityDialog dlg;
  QString strName;

  dlg.setCaption("New Profile");
  if(dlg.exec()) {
    strName = dlg.getName();
    if(strName.isEmpty())
      return;
    if(entityExists(strName)) {
      QMessageBox::information(this, "Duplicate Profile", "There is already a Profile with that Name.", QMessageBox::Ok | QMessageBox::Default);
      return;
    }
    pCurrentEntity = addEntity(strName);
   
  }
  m_pDeleteProfile->setEnabled(true);
  m_pRenameProfile->setEnabled(true);
  slotSave("netinventory.dat");
  return;
}

void CNetInventory::slotRenameEntity()
{
  QString strName;
  ASSERT(pCurrentEntity);
  if(!pCurrentEntity)
    return;

  CNewEntityDialog dlg(m_pEntityCombo->currentText());
  if(dlg.exec()) {
    strName = dlg.getName();
    if(strName.isEmpty())
      return;
    if(entityExists(strName)) {
      QMessageBox::information(this, "Duplicate Entity", "There is already a Profile with that Name.", QMessageBox::Ok |  QMessageBox::Default);
      return;
    }
    pCurrentEntity->newName(strName);
    m_pEntityCombo->changeItem(strName, m_pEntityCombo->currentItem());

    // 
    m_pEntityCombo->update();
    slotSave("netinventory.dat");
    return;
  
  }
}
void CNetInventory::slotAddProduct()
{

  ASSERT(pCurrentEntity);

  if(!pCurrentEntity)
    return;
  // General vulnerability category
  if(m_pVulnTree->selectedItem()) {
    CVulnCatItem *item = (CVulnCatItem *)  m_pVulnTree->selectedItem();
    CVulnCatItem *pitem = (CVulnCatItem *) item->parent();
    if(pitem) {
      pCurrentEntity->addProduct(pitem->text(0), pitem->getID(), item->text(0), item->getID(), m_pVulnList);
    }
    else {
      pCurrentEntity->addProduct(item->text(0), item->getID(), "ALL Products", 0, m_pVulnList);
    }
  }

  // Custom category
  if(m_pCustomList->selectedItem()) {
    CCustomItem *item = (CCustomItem *) m_pCustomList->selectedItem();
    pCurrentEntity->addProduct("CUSTOM", 0, item->text(0), item->getID(), m_pVulnList);
  }

  slotSave("netinventory.dat");

  // Enable clear and remove buttons
  m_pClearButton->setEnabled(true);
  if(m_pVulnList->selectedItem())
    m_pRemoveButton->setEnabled(true);
}

void CNetInventory::slotRemoveProduct()
{
  ASSERT(pCurrentEntity);
  if(!pCurrentEntity)
    return;
  QListViewItem *pItem = m_pVulnList->selectedItem();
  if(!pItem)
    return;

  pCurrentEntity->removeByItemPtr(pItem);
  slotSave("netinventory.dat");

  // If we removed the last item, disable clear and remove buttons
  if(m_pVulnList->childCount() == 0) {
    m_pRemoveButton->setEnabled(false);
    m_pClearButton->setEnabled(false);
  }
  // If an item is no longer selected (because it was removed) disable remove button
  if(!m_pVulnList->selectedItem())
    m_pRemoveButton->setEnabled(false);
}

void CNetInventory::slotClearProducts()
{
  ASSERT(pCurrentEntity);
  if(!pCurrentEntity)
    return;

  pCurrentEntity->removeAll(m_pVulnList);
  slotSave("netinventory.dat");

  // Disable clear and remove buttons
  m_pRemoveButton->setEnabled(false);
  m_pClearButton->setEnabled(false);
}

//
// Display a tree view of all profiles and associated products in the current profile list
//
void CNetInventory::setAllByProfile()
{
  CNetworkEntity *pEntity;
  pCurrentEntity = NULL;
  m_pVulnList->clear();
  m_pVulnList->setRootIsDecorated(true);

  for(pEntity = m_Entities.first(); pEntity; pEntity = m_Entities.next()) {
    QListViewItem *pEntityListItem = new QListViewItem(m_pVulnList, pEntity->getName());
    pEntity->populateEntity(pEntityListItem);
    pEntityListItem->setOpen(true);
  }
    
  m_pDeleteProfile->setEnabled(false);
  m_pRenameProfile->setEnabled(false);
  
}
//
// Display a flat list of all of the products in all of the profiles in the current profile list
//
void CNetInventory::setAllCombined()
{
  CNetworkEntity *pEntity;
  pCurrentEntity = NULL;
  m_pVulnList->clear();
  m_pVulnList->setRootIsDecorated(false);

  for(pEntity = m_Entities.first(); pEntity; pEntity = m_Entities.next()) 
    pEntity->populateCombined(m_pVulnList);
    
}
void CNetInventory::slotChangeEntity(int i)
{
  // Index 0 is 'All (By Profile)'
  if(i == 0) {
    // Disable Add '->', Remove '<-' and Clear '<<'  buttons
    // Disable Rename/Delete profile buttons
    m_pAddButton->setEnabled(false);
    m_pRemoveButton->setEnabled(false);
    m_pClearButton->setEnabled(false);
    m_pDeleteProfile->setEnabled(false);
    m_pRenameProfile->setEnabled(false);
    setAllByProfile();
    return;
  }
  // Index 1 is 'All (Combined)'
  if(i == 1) {
    // Disable Add '->', Remove '<-' and Clear '<<' buttons
    // Disable Rename/Delete profile buttons
    m_pAddButton->setEnabled(false);
    m_pRemoveButton->setEnabled(false);
    m_pClearButton->setEnabled(false);
    m_pDeleteProfile->setEnabled(false);
    m_pRenameProfile->setEnabled(false);
    setAllCombined();
    return;
  }

  m_pVulnList->setRootIsDecorated(false);
  CNetworkEntity *pEntity = m_Entities.first();

  while(pEntity) { 
    if(pEntity->getIndex() == i)
      break;
    pEntity = m_Entities.next();
  }

  if(!pEntity) {
    qWarning("Could not find Entity for index");
    return;
  }

  pEntity->populateListview(m_pVulnList);
  pCurrentEntity = pEntity;
  slotSave("netinventory.dat");

  // Enable Add '->' button if something on either the product or custom list is selected
  
  if(m_pVulnTree->selectedItem() || m_pCustomList->selectedItem())
    m_pAddButton->setEnabled(true);

  // Enable Clear '<<' button if there are any items on the current entity list
  if(m_pVulnList->childCount() > 0) {
    m_pClearButton->setEnabled(true);

    // Enable Remove '<-' button if there is a selected item on this list
    if(m_pVulnList->selectedItem())
      m_pRemoveButton->setEnabled(true);
  }
  // Enable Delete/Rename profile buttons
  m_pDeleteProfile->setEnabled(true);
  m_pRenameProfile->setEnabled(true);
    
}


void CNetInventory::slotDeleteEntity()
{
  ASSERT(pCurrentEntity);
  if(!pCurrentEntity)
    return;

  CNetworkEntity *pEntity;
  QString strMessage;
  strMessage.sprintf("You are about to delete the profile named '%s'\n\nPress Ok to continue.", pCurrentEntity->getName().ascii());
  if(  QMessageBox::warning(this, "Confirm Delete Profile", strMessage, "Cancel", "Ok", NULL, 0) == 0)
    return;

  // Remove item from combo box and set correct combo box index for each of the remaining entities
  m_pEntityCombo->removeItem(pCurrentEntity->getIndex());
  m_Entities.remove(pCurrentEntity);

  for(int n = 2; n < m_pEntityCombo->count(); n++) {
    for(pEntity = m_Entities.first(); pEntity; pEntity = m_Entities.next()) {
      if(m_pEntityCombo->text(n) == pEntity->getName())
	pEntity->setIndex(n);
    }
  }

  
  delete  pCurrentEntity;
  
  pCurrentEntity = NULL;
  m_pEntityCombo->setCurrentItem(0);
  setAllByProfile();

  slotSave("netinventory.dat");
 
  
}

  
void CNetInventory::populateInventory(QString &strContent)
{
  QString strInventory, strEntity, strName, strProduct, strProductID, strVendorID;
  QString strCustom, strCustomItem, strCustomName, strCustomKeywords, strCustomID;
  int nVendorID, nProductID, nCustomID;
  CVulnCatItem *item, *parent;

  strTimestamp.truncate(0);
  CWWWServer::xmlExtract("timestamp", strContent, strTimestamp);
  if(strTimestamp.isEmpty())
    strTimestamp = "0";
  // First  process custom category information
  CWWWServer::xmlExtract("customitems", strContent, strCustom);
  while(CWWWServer::xmlExtract("custom", strCustom, strCustomItem)) {
    CWWWServer::xmlExtract("name", strCustomItem, strCustomName);
    CWWWServer::xmlExtract("keywords", strCustomItem, strCustomKeywords);
    CWWWServer::xmlExtract("id", strCustomItem, strCustomID);

    if(strCustomName.isEmpty() || strCustomKeywords.isEmpty())
      continue;
    nCustomID = strCustomID.toInt();
    if(nCustomID == 0)
      continue;
    new CCustomItem(m_pCustomList, strCustomName, strCustomKeywords, nCustomID);
  }
	
  // Now read in network inventory
  CWWWServer::xmlExtract("netinventory", strContent, strInventory);
  while(CWWWServer::xmlExtract("entity", strInventory, strEntity)) {
    CWWWServer::xmlExtract("name", strEntity, strName);
    pCurrentEntity = addEntity(strName);

    while(CWWWServer::xmlExtract("product", strEntity, strProduct)) {
      CWWWServer::xmlExtract("vid", strProduct, strVendorID);
      CWWWServer::xmlExtract("pid", strProduct, strProductID);
      nVendorID = strVendorID.toInt();
      nProductID = strProductID.toInt();
      // Add 'ALL Products' for a certain category
      if(nProductID == 0) {
	if(!(parent = id2VulnItem(nVendorID, false)))
	  continue;
	pCurrentEntity->addProduct(parent->text(0), parent->getID(), "ALL Products", 0, m_pVulnList);
	continue;
      }

      // Add a custom category
      if(nVendorID == 0) {
	CCustomItem *citem;
	if(!(citem = id2CustomItem(nProductID)))
	  continue;
	pCurrentEntity->addProduct("CUSTOM", 0, citem->text(0), citem->getID(), m_pVulnList);
	continue;
      }
	
      // Add an individual product
      if(!(item = id2VulnItem(nProductID, true))) {
	continue;
      }
      parent = (CVulnCatItem *) item->parent();
      if(!parent)
	continue;
      pCurrentEntity->addProduct(parent->text(0), parent->getID(), item->text(0), item->getID(), m_pVulnList);
    }

  }


    
  return;
}

void CNetInventory::slotLoad(QString strFilename)
{
  QString strBuffer, strTmp;
  QFile f(strPagerDir + strFilename);
  if(!f.open(IO_ReadOnly))
    return;
  
  while(!f.atEnd()) {
    f.readLine(strTmp, 1024);
    strBuffer += strTmp;
  }
  f.close();
  populateInventory(strBuffer);
  return;
}
void CNetInventory::slotSave(QString strFilename)
{
  
  QFile f(strPagerDir + strFilename);
  QString strBuffer;
  
  if(f.open(IO_Truncate | IO_WriteOnly)) {
    QTextStream t(&f);

    t << "<timestamp>" << strTimestamp << "</timestamp>" << endl;
    t << "<customitems>" << endl;
    for(CCustomItem *pItem = (CCustomItem *)m_pCustomList->firstChild(); pItem; pItem = (CCustomItem *)pItem->nextSibling())
      pItem->saveSelf(t);
    t << "</customitems>" << endl;

    t << "<netinventory>" << endl;
    for(CNetworkEntity *pEntity = m_Entities.first(); pEntity; pEntity = m_Entities.next()) {
      pEntity->saveSelf(t);
    }
    t << "</netinventory>" << endl;

    f.close();
  }
  return;
}

void CNetInventory::slotNewCustom()
{
  CCustomDialog dlg(0, 0, this);
  QString strName, strKeywords;

  if(dlg.exec()) {
    strName = dlg.getName();
    strKeywords = dlg.getKeywords();
    if(strName.isEmpty()) {
      QMessageBox::warning(this, "New Custom Category", "You must specify a name for the new category.");
      return;
    }
    if(strKeywords.isEmpty()) {
      QMessageBox::warning(this, "New Custom Category", "You must specify keywords for the new category.");
      return;
    }
    new CCustomItem(m_pCustomList, strName, strKeywords, getCustomID());
    slotSave("netinventory.dat");
    return;
  }

  return;
}

// 
// return the lowest integer ID that is not currently being used by a custom category
//
int CNetInventory::getCustomID() {

  CCustomItem *item;
  int ID = 1;


  for(item = (CCustomItem *)m_pCustomList->firstChild(); item; /* */) {
    if(item->getID() == ID) {
      ID++;
      item = (CCustomItem *)m_pCustomList->firstChild();
    }
    else {
      item = (CCustomItem *)item->nextSibling();
    }
  }
  

  return ID;
}

// When something on Vuln list is selected clear selection on Custom list
void CNetInventory::slotVulnSelected()
{
  // Clear selection generates selectionChanged()
  if(!m_pVulnTree->selectedItem())
    return;

  m_pCustomList->clearSelection();

  // Disable buttons for manipulating currently selected custom item
  m_pDelCustomButton->setEnabled(false);
  m_pEditCustomButton->setEnabled(false);

  // Enable Add '->' button if a profile is selected
  if(m_pEntityCombo->currentItem() > 1)
    m_pAddButton->setEnabled(true);

}

void CNetInventory::slotCustomSelected()
{
  // clearSelection() generates selectionChanged()  -- ensure that these two functions don't call each other forever
  if(!m_pCustomList->selectedItem())
    return;

  m_pVulnTree->clearSelection();
  // Enable buttons for manipulating currently selected custom item
  m_pDelCustomButton->setEnabled(true);
  m_pEditCustomButton->setEnabled(true);

  // Enable Add '->' button if a profile is selected
  if(m_pEntityCombo->currentItem() > 1)
    m_pAddButton->setEnabled(true);
}
  

void CNetInventory::slotVulnListSelected()
{
  if(m_pEntityCombo->currentItem() > 1 && m_pVulnList->selectedItem())
      m_pRemoveButton->setEnabled(true);
}

void CNetInventory::slotDeleteCustom()
{
  CCustomItem *customItem = (CCustomItem *)m_pCustomList->selectedItem();
  ASSERT(customItem);
  if(!customItem)
    return;

  int id = customItem->getID();

  // Search all profiles for this custom id (ie Vendor == 0 && Product == id)
  for(CNetworkEntity *pEntity = m_Entities.first(); pEntity; pEntity = m_Entities.next()) {
    if(pEntity->itemExists(0, id)) {
      if( QMessageBox::warning(this, "Confirm Delete Custom", 
			       "One or more of your network profiles contains the custom category you are about to delete.\n\nPress Ok to continue.", 
			       "Cancel", "Ok", NULL, 0) == 0)
	return;
      break;
    }
  }

  //
  // Remove any instances of this item in the user profiles/entities
  //
  for(CNetworkEntity *pEntity = m_Entities.first(); pEntity; pEntity = m_Entities.next())
    pEntity->removeByID(0, id);

  delete customItem;
  // Should be nothing selected now so disable some buttons
  m_pDelCustomButton->setEnabled(false);
  m_pEditCustomButton->setEnabled(false);
  m_pAddButton->setEnabled(false);

  slotChangeEntity(m_pEntityCombo->currentItem());

  slotSave("netinventory.dat");

  
  
}

void CNetInventory::slotEditCustom()
{
  QString strOldName;
  CCustomItem *customItem = (CCustomItem *)m_pCustomList->selectedItem();
  ASSERT(customItem);
  if(!customItem)
    return;

  CCustomDialog dlg(customItem->text(0), customItem->text(1), this);
  QString strName, strKeywords;

  if(dlg.exec()) {
    strName = dlg.getName();
    strKeywords = dlg.getKeywords();
    if(strName.isEmpty()) {
      QMessageBox::warning(this, "Edit Custom Category", "You must specify a name for the category.");
      return;
    }
    if(strKeywords.isEmpty()) {
      QMessageBox::warning(this, "Edit Custom Category", "You must specify keywords for the category.");
      return;
    }
    strOldName = customItem->text(0);

    if(strOldName != strName) 
      // replace name in all profiles containing this custom category
      for(CNetworkEntity *pEntity = m_Entities.first(); pEntity; pEntity = m_Entities.next()) 
	pEntity->customRename(customItem->getID(), strName);

    customItem->setText(0, strName);
    customItem->setText(1, strKeywords);

    setAllByProfile();
    slotSave("netinventory.dat");
  }
}


void CNetInventory::processVulns()
{
  QString strTmp, strBuffer, strVuln, strOutput;
  QTextStream os(strOutput, IO_WriteOnly);
  CVulnerability theVuln;
  bool bIsAnythingVulnerable;

  QFile f(::strPagerDir + "vulnpoll.dat");
  if(!f.open(IO_ReadOnly))
    return;
  
  while(!f.atEnd()) {
    f.readLine(strTmp, 1024);
    strBuffer += strTmp;
  }

  while(CWWWServer::xmlExtract("vuln", strBuffer, strVuln)) {
    if(!CWWWServer::xmlExtract("bid", strVuln, theVuln.strBID)) continue;
    if(!CWWWServer::xmlExtract("title", strVuln, theVuln.strTitle)) continue;
    if(!CWWWServer::xmlExtract("object", strVuln, theVuln.strObject)) continue;
    if(!CWWWServer::xmlExtract("class", strVuln, theVuln.strClass)) continue; 
    if(!CWWWServer::xmlExtract("remote", strVuln, theVuln.strRemote)) continue; 
    if(!CWWWServer::xmlExtract("local", strVuln, theVuln.strLocal)) continue; 
    if(!CWWWServer::xmlExtract("published", strVuln, theVuln.strPublished)) continue; 
    if(!CWWWServer::xmlExtract("updated", strVuln, theVuln.strUpdated)) continue; 
    if(!CWWWServer::xmlExtract("vulnerable", strVuln, theVuln.strVulnerable)) continue; 
    if(!CWWWServer::xmlExtract("notvulnerable", strVuln, theVuln.strNotVulnerable)) continue; 
    if(!CWWWServer::xmlExtract("discussion", strVuln, theVuln.strDiscussion)) continue; 
    if(!CWWWServer::xmlExtract("exploit", strVuln, theVuln.strExploit)) continue; 
    if(!CWWWServer::xmlExtract("solution", strVuln, theVuln.strSolution)) continue; 
    if(!CWWWServer::xmlExtract("credit", strVuln, theVuln.strCredit)) continue; 

    // If vulnerability is newer than strTimestamp, update strTimestamp
    if(theVuln.strUpdated > strTimestamp)
      strTimestamp = theVuln.strUpdated;

    bIsAnythingVulnerable = false;
    strTmp.truncate(0);
    for(CNetworkEntity *pEntity = m_Entities.first(); pEntity; pEntity = m_Entities.next()) {
      if(pEntity->vulnerableByProduct(IDList(theVuln.strVulnerable, true)) ||
	 pEntity->vulnerableByVendor(IDList(theVuln.strVulnerable, false)) ||
	 pEntity->vulnerableByCustom(IDList(theVuln.strVulnerable, true), m_pVulnTree, m_pCustomList, theVuln.strTitle)) {
	if(!bIsAnythingVulnerable) {
	  bIsAnythingVulnerable = true;
	  strTmp += pEntity->getName();
	}
	else {
	  strTmp += ", ";
	  strTmp += pEntity->getName();
	}

      }
    }
    if(bIsAnythingVulnerable) {
      os << endl << "The profile(s) [" << strTmp << "] in your network inventory was found to be vulnerable to the following issue:" << endl << endl;
      os << theVuln.strTitle << endl << endl;
      os << "Published: " << theVuln.strPublished << endl;
      os << "Updated: " << theVuln.strUpdated << endl << endl;
      os << "Vulnerable:" << endl << products2Names(theVuln.strVulnerable) << endl;
      os << "Not Vulnerable:" << endl << products2Names(theVuln.strNotVulnerable) << endl;

      os << "BugTraq ID: " << theVuln.strBID << endl;
      os << "Class: " << theVuln.strClass << endl;
      os << "Remote: " << theVuln.strRemote  << endl << "Local: " << theVuln.strLocal << endl;

      os << "Discussion" << endl << theVuln.strDiscussion << endl;
      os <<  endl << "Exploit:" << endl <<  endl << theVuln.strExploit << endl;
      os << endl << "Solution:" << endl << endl << theVuln.strSolution << endl;
      os << endl << "Credit:" << endl << endl << theVuln.strCredit << endl;
      os << endl << endl << "--------------------------------------------------------------------------" << endl << endl;

      if(m_pConfig->getBriefNotify())
	notifyBrief(theVuln, strTmp);

      if(m_pConfig->getVulnPopup())
	notifyPopup(theVuln, strTmp);
    }
    

  }
  if(!strOutput.isEmpty() && m_pConfig->getVerboseNotify())
    notifyVerbose(strOutput);

  slotSave("netinventory.dat");
}

void CNetInventory::notifyPopup(CVulnerability &vuln, QString &strName)
{
  QString strBuffer = "The network profile(s) [" + strName + "] was found to be vulnerable to the following issue:\n\n";
  strBuffer += "Bugtraq ID [" + vuln.strBID + "] : " + vuln.strTitle;
  
  QMessageBox::information(this, "SecurityFocus Pager Vulnerability Notification", strBuffer, QMessageBox::Ok | QMessageBox::Default);
}

void CNetInventory::notifyVerbose(QString &strOut)
{
  QString strCommand = m_pConfig->getMailer() + " " + m_pConfig->getVerboseEmail();
  FILE *pF = popen(strCommand, "w");
  if(pF == NULL) {
    cerr << "Could not open pipe to mailer with command line: " << strCommand << endl;
    return;
  }
  QTextStream o(pF, IO_WriteOnly);

  o << strOut;

  pclose(pF);
  
}
void CNetInventory::notifyBrief(CVulnerability &vuln, QString &strName)
{
  QString strCommand = m_pConfig->getMailer() + " " + m_pConfig->getBriefEmail();
  FILE *pF = popen(strCommand, "w");
  if(pF == NULL) {
    cerr << "Could not open pipe to mailer with command line: " << strCommand << endl;
    return;
  }
  QTextStream o(pF, IO_WriteOnly);
  
  o << "SecurityFocus Pager Profile [" << strName << "] is vulnerable to BID [" << vuln.strBID << "] : " << vuln.strTitle << endl;

  pclose(pF);
}

QString CNetInventory::products2Names(QString strVulnerable)
{
  QString strPair;
  QString strProduct, strVendor;
  QString strNames;
  CVulnCatItem *product, *vendor;

  int comma, colon;
  while( (comma = strVulnerable.find(',', 0)) != -1) {
    strPair = strVulnerable.left(comma);
    // nuke up to the comma
    strVulnerable.remove(0, comma + 1);
    if( ( colon = strPair.find(':', 0)) == -1)
      continue;

    strVendor = strPair.left(colon);
    // erase everything up to and including colon
    strPair.remove(0, colon + 1);
    strProduct = strPair;

    product = id2VulnItem(strProduct.toInt(), true);
    vendor = id2VulnItem(strVendor.toInt(), false);
    if(!(product && vendor))
      continue;
    strNames += vendor->text(0) + " " + product->text(0) + "\n";
  }
  strPair = strVulnerable;
  if( ( colon = strPair.find(':', 0)) != -1) {
    strVendor = strPair.left(colon);
    strPair.remove(0, colon + 1);
    strProduct = strPair;
    product = id2VulnItem(strProduct.toInt(), true);
    vendor = id2VulnItem(strVendor.toInt(), false);
    if(product && vendor)
      strNames += vendor->text(0) + " " + product->text(0) + "\n";
  }
  if(strNames.isEmpty())
    strNames += "N/A";
  return strNames;
}

QList<int> & CNetInventory::IDList(QString strVulnerable, bool products)
{
  QString strPair;
  QString strProduct, strVendor;
  int comma, colon;
  static QList <int>list;

  list.setAutoDelete(true);
  list.clear();

  while( (comma = strVulnerable.find(',', 0)) != -1) {
    strPair = strVulnerable.left(comma);
    // nuke up to the comma
    strVulnerable.remove(0, comma + 1);

    if( (colon = strPair.find(':', 0)) == -1)
      continue;
    
    strVendor = strPair.left(colon);
    // erase everything up to and including colon
    strPair.remove(0, colon + 1);
    strProduct = strPair;
    if(products) {
      list.append(new int(strProduct.toInt()));
    }
    else {
      list.append(new int(strVendor.toInt()));
    }

  }

  strPair = strVulnerable;
  if( (colon = strPair.find(':', 0)) != -1) {
    strVendor = strPair.left(colon);
    strPair.remove(0, colon + 1);
    strProduct = strPair;
    if(products) {
      list.append(new int(strProduct.toInt()));
    }
    else {
      list.append(new int(strVendor.toInt()));
    }
  }

  return list;
}
    
    







