#ifndef INCLUDE_NETINVENTORY_H
#define INCLUDE_NETINVENTORY_H
#include <qwidget.h>
#include <qlist.h>
#include <qstring.h>
#include "networkentity.h"
#include "wwwserver.h"
#include "vulnerability.h"

class CVulnCatItem;
class CCustomItem;
class QListView;
class QComboBox;
class QPushButton;
class CConfig;
class CNetInventory : public QWidget {

Q_OBJECT
 public:
  CNetInventory(CWWWServer *www, CConfig *pConfig, QWidget *parent = 0, const char *name= 0);
  void processVulns();
  int getVulnVersion() { return m_nVersion; }
  QString getVulnTimestamp() { return strTimestamp; }
  bool anyProfiles() const { return (!m_Entities.isEmpty()); }
 protected:
  void setup();
  int m_nVersion;
  QString strTimestamp; // most recent vulnerability timestamp

  CWWWServer *wwwServer;
  // Buttons galore
    
  QPushButton *m_pNewCustomButton;
  QPushButton *m_pDelCustomButton;
  QPushButton *m_pEditCustomButton;
  QPushButton *m_pAddButton;
  QPushButton *m_pRemoveButton;
  QPushButton *m_pClearButton;
  QPushButton *m_pDeleteProfile;
  QPushButton *m_pRenameProfile;
    
  QListView * m_pVulnTree;
  QListView * m_pCustomList;
  QListView * m_pVulnList;
  QComboBox * m_pEntityCombo;

  CConfig *m_pConfig;
  void populateVulnTree();
  CNetworkEntity *addEntity(QString strName);
  bool entityExists(QString strName);
  CVulnCatItem *id2VulnItem(int id, bool descend);
  CCustomItem *id2CustomItem(int id);
  QString products2Names(QString strVulnerable);

  QList <CNetworkEntity> m_Entities;
  CNetworkEntity *pCurrentEntity;
  void populateInventory(QString &strContent);
  int getCustomID();
  void setAllByProfile();
  void setAllCombined();
  QList <int> &IDList(QString strVulnerable, bool products);

  void notifyVerbose(QString &strOut);
  void notifyBrief(CVulnerability &vuln, QString &strName);
  void notifyPopup(CVulnerability &vuln, QString &strName);
public slots:
  void slotLoadVulnTree();
 void slotNewEntity(); 
 void slotAddProduct();
 void slotRemoveProduct();
 void slotClearProducts();
 void slotChangeEntity(int i);
 void slotSave(QString strFilename);
 void slotLoad(QString strFilename);
 void slotDeleteEntity();
 void slotNewCustom();
 void slotVulnSelected();
 void slotCustomSelected();
 void slotVulnListSelected();
 void slotRenameEntity();
 void slotDeleteCustom();
 void slotEditCustom();
};



#endif


