#include "networkentity.h"
#include "customitem.h"
#include "vulncatitem.h"
#include <qlistview.h>
#include <iostream>
CNetworkEntity::CNetworkEntity(QString name, int index) : strName(name), nIndex(index) {
  // automatically delete items when we remove them
  m_products.setAutoDelete(TRUE);
}
CNetworkEntity::~CNetworkEntity()
{
  m_products.clear();
}
// returns true if product does not already exist in list
bool CNetworkEntity::addProduct(QString vendor, int vid, QString product, int pid, QListView *pList) {
  CProductItem *item;
  // Does this product already exist?
  for(item = m_products.first(); item != 0; item=m_products.next()) {
    if(item->nVendorID == vid && item->nProductID == 0) // vendor already exists with products == ALL
      return false;
    if(item->nVendorID == vid && item->nProductID == pid) // specific vendor and product already exist
      return false;
  }
  // If user adds an entire vendor remove any redundant single products

  if(pid == 0) 
    for(item = m_products.first(); item != 0;) {
      if(item->nVendorID == vid && item->nProductID) {
	// remove item from listview control then from list
	delete item->pListItem;
	m_products.remove();
	item = m_products.current();
      }
      else
	item = m_products.next();
     }
  
  QListViewItem  *pListItem = new QListViewItem(pList, vendor, product);
  
  m_products.append(new CProductItem(vendor, vid, product, pid, pListItem));
  
  return true;
}

void CNetworkEntity::removeByItemPtr(QListViewItem *pItem)
{
  CProductItem *item;

  
  for(item = m_products.first(); item != 0; item=m_products.next()) 
    if(item->pListItem == pItem) {
      m_products.remove();      // remove current item
      delete pItem;            // remove from ListView
      return;
    }
}
    
void CNetworkEntity::removeAll(QListView *pList)
{
  m_products.clear();
  pList->clear();
}

void CNetworkEntity::populateListview(QListView *pList)
{

  QListViewItem *listItem;
  CProductItem *product;
  
  pList->clear();

  for(product = m_products.first(); product; product = m_products.next()) {
    listItem = new QListViewItem(pList, product->strVendorName, product->strProductName);
    product->pListItem = listItem;
  }
}

//
// invoked per entity for 'All (By Profile)'
//
void CNetworkEntity::populateEntity(QListViewItem *pListEntity)
{
  CProductItem *product;
  for(product = m_products.first(); product; product = m_products.next()) 
    new QListViewItem(pListEntity, product->strVendorName, product->strProductName);
}

// 
// invoked per entity for 'All (Combined)'
//
void CNetworkEntity::populateCombined(QListView *pList)
{
  CProductItem *product;
  QListViewItem *item;

  for(product = m_products.first(); product; product = m_products.next()) {
    for(item = pList->firstChild(); item; item = item->nextSibling()) 
      if( (product->strVendorName == item->text(0)) && (product->strProductName == item->text(1)) ) 
	break;              // List already contains this item;

    if(!item)
      new QListViewItem(pList, product->strVendorName, product->strProductName);
    
  }
}

//
// returns true if a product exists in this entity with the specified vendor and product id
//
bool CNetworkEntity::itemExists(int vid, int pid)
{
  CProductItem *product;
  for(product = m_products.first(); product; product = m_products.next())
    if( (vid == product->nVendorID) && (pid == product->nProductID) )
      return true;
  return false;
}

//
// Removes any product with matching vendor and product ids
//
void CNetworkEntity::removeByID(int vid, int pid)
{
  CProductItem *product;
  for(product = m_products.first(); product; product = m_products.next()) 
    if( (vid == product->nVendorID) && (pid == product->nProductID) ) {
      // remove product from array
      m_products.remove(product);
    }
}
      

void CNetworkEntity::saveSelf(QTextStream &t)
{
  t << "\t<entity>" << endl;
  t << "\t\t<name>" << strName << "</name>" << endl;
  for(CProductItem *pProduct = m_products.first(); pProduct; pProduct = m_products.next()) {
    t << "\t\t<product>" << endl;
    t << "\t\t\t<vid>" << pProduct->nVendorID << "</vid>" << endl;
    t << "\t\t\t<pid>" << pProduct->nProductID << "</pid>" << endl;
    t << "\t\t</product>" << endl;
  }
  t << "\t</entity>" << endl;
}



bool CNetworkEntity::vulnerableByProduct(QList <int>productList) 
{
  for(CProductItem *pProduct = m_products.first(); pProduct; pProduct = m_products.next()) {
    if(!pProduct->nVendorID) // nVendorID == 0 means custom category
      continue;
    if(!pProduct->nProductID) // nProductID == 0 means entire vendor
      continue;
    for(int *pN = productList.first(); pN; pN = productList.next()) 
      if(pProduct->nProductID == *pN)
	return true;
  }
  // not found
  return false;
}

bool CNetworkEntity::vulnerableByVendor(QList <int>vendorList)
{
  for(CProductItem *pProduct = m_products.first(); pProduct; pProduct = m_products.next()) {
    if(!pProduct->nVendorID) // nVendorID == 0 means it is a custom category
      continue;
    if(pProduct->nProductID) // nProductID != 0 means that this is not an entire vendor
      continue;
    
    for(int *pN = vendorList.first(); pN; pN = vendorList.next()) 
      
      if(pProduct->nVendorID == *pN)
	return true;
  }

  // not found
  return false;
}

void CNetworkEntity::customRename(int id, QString &newName)
{
  for(CProductItem *pProduct = m_products.first(); pProduct; pProduct = m_products.next()) {
    if(pProduct->nVendorID) // if nVendorID isn't 0 this isn't a custom category
      continue;
    if(pProduct->nProductID != id)
      continue;
    pProduct->strProductName = newName;
  }
  return;
}

bool CNetworkEntity::vulnerableByCustom(QList <int>productList, QListView *pVulnTree, QListView *pCustomList,  QString strTitle)
{
  QString strKeywords, strWord;
  int comma;

  for(CProductItem *pProduct = m_products.first(); pProduct; pProduct = m_products.next()) {
    if(pProduct->nVendorID) // if nVendorID isn't 0 this isn't a custom category so skip it
      continue;
    for(CCustomItem *pCustom = (CCustomItem *)pCustomList->firstChild(); pCustom; pCustom = (CCustomItem *)pCustom->nextSibling()) {
      if(pCustom->getID() != pProduct->nProductID)
	continue;
      //
      // pProduct is an instance of pCustom in this entity's profile
      //
      strKeywords = pCustom->text(1);
      while( (comma = strKeywords.find(',', 0)) != -1) {
	strWord = strKeywords.left(comma);

	strKeywords.remove(0, comma + 1);
	if(vulnerableByKeyword(strWord, productList, pVulnTree)) 
       	  return true;
	
	if(strTitle.contains(strWord, false)) 
	  return true;
	
      }
      if(vulnerableByKeyword(strKeywords, productList, pVulnTree)) 
	return true;

      if(strTitle.contains(strKeywords, false)) 
	return true;
    }
  }
  return false;
}
bool CNetworkEntity::vulnerableByKeyword(QString strWord, QList <int>productList, QListView *pVulnTree)
{
  // Ok, here's the plan.  We need to find each item in 'productList' on the Vulnerability Tree.  Then we'll suck out the strings for both the 
  // vendor and product name and grep for strWord

  CVulnCatItem *vendor, *product;

  for(int *pN = productList.first(); pN; pN = productList.next()) {
    for(vendor = (CVulnCatItem *)pVulnTree->firstChild(); vendor; vendor = (CVulnCatItem *)vendor->nextSibling()) 
      for(product = (CVulnCatItem *)vendor->firstChild(); product; product = (CVulnCatItem *)product->nextSibling()) 

	if(product->getID() == *pN)  // Found the product, look for strWord in vendor->text(0) and product->text(0)
	  if(vendor->text(0).contains(strWord, false) || product->text(0).contains(strWord, false)) 
	    return true;
  }
  return false;
}



