#include "options.h"
#include <qlayout.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qslider.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qframe.h>
#include <qstring.h>
#include <qfiledialog.h>
#include <qcombobox.h>
#include <qfile.h>

#include <sys/types.h>
#include <dirent.h>
#include <iostream>
COptions::COptions(QWidget *parent, const char *name)
  :QWidget(parent, name), nItems(nInitialItems), nPollInterval(nInitialInterval)
{
  setup();
}

void COptions::setup() 
{

  // Top level layout
  QHBoxLayout *h1 = new QHBoxLayout(this);

   QVBoxLayout *v1 = new QVBoxLayout();
   QVBoxLayout *v2 = new QVBoxLayout();




  QVGroupBox *numItemsBox = new QVGroupBox("Maximum Articles", this);
  numItemsBox->setAlignment(AlignHCenter);


  QVGroupBox *pollIntervalBox = new QVGroupBox("Server Polling Interval", this);
  pollIntervalBox->setAlignment(AlignHCenter);


  new QLabel(" ", numItemsBox);
  numItemsSlider = new QSlider(nMinItems, nMaxItems, nStep, nInitialItems,
					QSlider::Horizontal, numItemsBox);
  numItemsLabel = new QLabel("0 Articles", numItemsBox);
  numItemsLabel->setAlignment(AlignHCenter | AlignTop);


  new QLabel("", pollIntervalBox);
  pollIntervalSlider = new QSlider(nMinInterval, nMaxInterval, nStep, nInitialInterval,
					   QSlider::Horizontal, pollIntervalBox);
  pollIntervalLabel = new QLabel("0 Minutes", pollIntervalBox);
  pollIntervalLabel->setAlignment(AlignHCenter | AlignTop);


  QVGroupBox *notificationBox = new QVGroupBox("Notification", this);
  
  checkFlashIcon = new QCheckBox("&Flashing Icon", notificationBox);
  checkPopup = new QCheckBox("&Pop up window", notificationBox);
  new QLabel("", notificationBox);


  QVGroupBox *skinBox = new QVGroupBox("Skin Select", this);
  skinCombo = new QComboBox(skinBox);
  populateSkinCombo();
  skinBox->setMinimumWidth(150);

  QVGroupBox *restartBox = new QVGroupBox("", this);
  checkConfigRestart = new QCheckBox("Restart Pager\non config", restartBox);

  QHGroupBox *browserBox = new QHGroupBox("Web Browser Path", this);
  browserBox->setMinimumWidth(350);
  QHBoxLayout *h2 = new QHBoxLayout();
  m_pBrowserEdit = new QLineEdit("", browserBox);
  QPushButton *p1 = new QPushButton("Browse", browserBox);

  h1->addLayout(v1);
  h1->addLayout(v2);
  
  v1->addWidget(pollIntervalBox, 1);
  v1->addWidget(numItemsBox, 1);
  v1->addLayout(h2, 2);
  
  v2->addWidget(notificationBox, 2);
  v2->addWidget(skinBox, 2);
  v2->addWidget(restartBox, 1);

  h2->addWidget(browserBox);
  
  // Signals
  QObject::connect(numItemsSlider, SIGNAL(valueChanged(int)), this, SLOT(slotNumItems(int)));
  QObject::connect(pollIntervalSlider, SIGNAL(valueChanged(int)), this, SLOT(slotPollInterval(int)));
  QObject::connect(p1, SIGNAL(clicked()), this, SLOT(slotChooseBrowser()));
  QObject::connect(skinCombo, SIGNAL(activated(int)), SLOT(slotSkinChange(int)));
  slotNumItems(nItems);
  slotPollInterval(nPollInterval);
}

void COptions::slotSkinChange(int)
{
  return;
}
void COptions::populateSkinCombo() {
  DIR *d;
  struct dirent *ent;
  QString strName, strLine;
  int n;

  d = opendir("/usr/lib/SecurityFocus/pagerskins");
  if(!d) {
    cerr << "Could not open skin directory" << endl;
    exit(1);
  }
  skinCombo->insertItem("No Skin");
  skinDirectories.append("");
  while((ent = readdir(d))) {
    if( (!strcmp(ent->d_name, ".")) || (!strcmp(ent->d_name, "..")))
      continue;

    QString strFile;
    QString strDirectory;
    strDirectory.sprintf("/usr/lib/SecurityFocus/pagerskins/%s", ent->d_name);
    strFile.sprintf("/usr/lib/SecurityFocus/pagerskins/%s/skin.ini",  ent->d_name);
    QFile f(strFile);

    strName = "";
    if(f.open(IO_ReadOnly)) {
      while(!f.atEnd() && strName.isEmpty()) {
	f.readLine(strLine, 1024);
	strLine.stripWhiteSpace();
	if( (n = strLine.find("name=", 0)) != -1) {
	  strName = strLine.mid(5);
	  strName = strName.stripWhiteSpace();
	}
      }
      f.close();
    }
    if(strName.isEmpty() != true) {
      skinCombo->insertItem(strName);
      skinDirectories.append(strDirectory);
    }
	
  }

  if(skinCombo->count() == 0) {
    cerr << "Could not find any skins in /usr/lib/SecurityFocus/pagerskins" << endl;
    exit(1);
  }

}


void COptions::slotNumItems(int n) {
  QString tmp;
  nItems = n;
  tmp.sprintf("%d Articles", n);
  numItemsLabel->setText(tmp);
}

void COptions::slotPollInterval(int n) {
  QString tmp;
  nPollInterval = n;
  tmp.sprintf("%d Minutes", n);
  pollIntervalLabel->setText(tmp);
}

void COptions::slotChooseBrowser() {
  QString strBrowser = QFileDialog::getOpenFileName("/");
  if(strBrowser.isEmpty())
    return;
  m_pBrowserEdit->setText(strBrowser);
}

void COptions::setSkin(const QString &strSkin)
{
  int i;

  for(i = 0; i < skinCombo->count(); i++) 
    if(strSkin == skinCombo->text(i)) {
      skinCombo->setCurrentItem(i);
      return;
    }
      
}






