#include "pager.h"
#include "wwwserver.h"
#include "categorytree.h"
#include "categoryitem.h"
#include "articleitem.h"
#include "config.h"
#include "vulnerability.h"
#include "searchdialog.h"

#include <qvbox.h>
#include <qpushbutton.h>
#include <qsplitter.h>
#include <qlistview.h>
#include <qtextbrowser.h>
#include <qapplication.h>
#include <qheader.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtimer.h>
#include <qtextstream.h>

#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/errno.h>

#include <iostream>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xmu/WinUtil.h>

#include <netdb.h>
#include <arpa/inet.h>
#include <string.h>
// Global
QString strPagerDir;
QString strBrowser;
extern bool bDebug;
extern bool bVulnTest;

void alarm_handler(int)
{
  cerr << "Timeout: Could not resolve server address" << endl;
  exit(1);
}
char *CPager::resolvServer(const char *name)
{

  alarm(10);
  signal(SIGALRM, alarm_handler);

  struct hostent *he = gethostbyname(name);
  alarm(0);
  signal(SIGALRM, SIG_DFL);

  if(!he) {
    cerr << "Failed to resolve address for server: " << name << endl;
    exit(1);
  }
  // We'll use the first address returned
  struct in_addr in;
  memcpy(&(in.s_addr), he->h_addr, 4);
  return inet_ntoa(in);
  
}

CPager::CPager(QWidget *parent, const char *name)
  : QVBox(parent, name), nVersion(0), bInit(false)
{

  bool bDoWizard = false;

  // Set up user's ~/.sfpager directory if it doesn't already exist		  
  bDoWizard = setupDirectory();
  
  theServer = new CWWWServer(resolvServer(szServerName));

  
  m_pConfig = new CConfig(theServer);
  m_pConfig->resize(300, 400);
  m_pConfig->setCaption("Pager Configuration");

  if(bDoWizard) {
    m_pConfig->setOptionsStuff(COptions::nInitialItems, COptions::nInitialInterval, wizard.getBrowser(), false, true, false);
    m_pConfig->setInventoryStuff("", "", wizard.getMailer(), false, false, false);
    m_pConfig->setSkin("Max");
    saveConfig(szConfigFile);
  }
  
  m_pSplitter = new QSplitter(Horizontal, this);
  m_pContentTree = new QListView(m_pSplitter);
  m_pContentTree->addColumn( QString("         "));
  m_pContentTree->setRootIsDecorated(true);
  m_pContentTree->header()->hide();
  m_pContentTree->setSorting(0, true);
  m_pContentTree->setSorting(1, false);


  m_pAbstract = new QTextBrowser(m_pSplitter);
  m_pAbstract->setTextFormat(RichText);
  m_pAbstract->setText(szEmptyAbstractText);
  

  int x1 = this->width() / 2;
  QValueList<int>sz;
  sz.append(x1);
  sz.append(this->width() - x1);
  m_pSplitter->setSizes(sz);
  QObject::connect(theServer, SIGNAL(categoryUpdateDone()), m_pConfig->getCategoryTree(), SLOT(slotLoadCategories()));
  QObject::connect(m_pConfig->getCategoryTree(), SIGNAL(categoriesLoaded()), this, SLOT(slotUpdateTopLevel()));

  
  // load configuration

  nPollInterval = COptions::nInitialInterval;
  nNumItems = COptions::nInitialItems;
  loadConfig(szConfigFile);
  if(m_pConfig->getSkinDirectory().isEmpty())
    currentSkin = NULL;
  else
    currentSkin = new CSkin(m_pConfig->getSkinDirectory());


  ::strBrowser = m_pConfig->getBrowserPath();
  QFile file(::strPagerDir + szCategoryFile);
  if(file.exists()) 
    m_pConfig->getCategoryTree()->loadCategories(szCategoryFile);
  else
    theServer->queueRequest(CWWWRequest::R_UPDATECATEGORY, NULL);
  
  m_pFlashTimer = new QTimer(this);
  QObject::connect(m_pFlashTimer, SIGNAL(timeout()), this, SLOT(slotFlashTimeout()));
  // display icon one
  emit newIcon(true);

  // start polling timer
 
  m_pTimer = new QTimer(this);
  QObject::connect(m_pTimer, SIGNAL(timeout()), this, SLOT(slotUpdateContent()));
  m_pTimer->start(nPollInterval * 1000 * 60);
  
  // start banner timer
  m_pBannerTimer = new QTimer(this);
  QObject::connect(m_pBannerTimer, SIGNAL(timeout()), this, SLOT(slotGetAd()));
  m_pBannerTimer->start(120 * 1000);
  theServer->queueRequest(CWWWRequest::R_FETCHAD, NULL);

  // connect some signals
  
  QObject::connect(theServer, SIGNAL(newContent(QString&)), this, SLOT(slotReceiveContent(QString &)));
  QObject::connect(theServer, SIGNAL(newAbstract(QString&)), this, SLOT(slotReceiveAbstract(QString &)));
  QObject::connect(m_pContentTree, SIGNAL(selectionChanged(QListViewItem *)), this, SLOT(slotArticleSelected(QListViewItem *)));
  QObject::connect(m_pContentTree, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(slotArticleDoubleClicked(QListViewItem *)));
  QObject::connect(theServer, SIGNAL(newVulns()), this, SLOT(slotReceiveVulns()));
  QObject::connect(theServer, SIGNAL(newAd(QString &, QString &)), this, SLOT(slotReceiveAd(QString &, QString &)));
  bInit = true;
  slotUpdateContent();

}
void CPager::applySkin()
{
  if(currentSkin)
    currentSkin->apply();
  else
    CSkin::applyNull();
  
}
//
// returns true if directory needed to be created
//
bool CPager::setupDirectory()
{

  struct stat sb;
  char *p = getenv("HOME");


  // This should be reasonable
  umask(0077);

  if(!p) 
    qFatal("Please set HOME environment variable");

  if( (stat(p, &sb) < 0) || !S_ISDIR(sb.st_mode))
    qFatal("Please set HOME environment variable correctly");

  char *buf = new char[strlen(p) + 25];

  sprintf(buf, "%s/.sfpager/", p);

  if( (stat(buf, &sb) < 0) && errno == ENOENT) {
  if(!wizard.exec()) 
    exit(0);
    if(mkdir(buf, 0700) < 0) 
      qFatal("Could not create configuration directory ~/.sfpager");
    ::strPagerDir = buf;
    delete [] buf;
    return true;
  }
  ::strPagerDir = buf;
  delete [] buf;
  return false;
}



void CPager::slotUpdateTopLevel() 
{
  m_pAbstract->setText(szEmptyAbstractText);
  if(bInit)
    saveArticles(szArticleFile);

  m_pConfig->getCategoryTree()->setActiveTopLevel(m_pContentTree);
  
  
  bInit = false;
  loadArticles(szArticleFile);
  bInit = true;

  m_pAbstract->setText(szEmptyAbstractText);
}

bool CPager::saveArticles(const QString &strFilename)
{
  QFile f(::strPagerDir + strFilename);
  CCategoryItem *item = (CCategoryItem *) m_pContentTree->firstChild();

  if(!f.open(IO_WriteOnly | IO_Truncate))
    return false;

  QTextStream t(&f);
  t << "<topnews>" << endl;

  while(item) {
    CArticleItem *article = (CArticleItem *)item->firstChild();

    while(article) {
      article->writeArticle(t);
      article = (CArticleItem *)article->nextSibling();
    }

    item = (CCategoryItem *)item->nextSibling();
  }

  t << "</topnews>" << endl;
  f.close();
  return true;
}

bool CPager::loadArticles(const QString &strFilename)
{
  QString strBuffer, strTmp;
  
  QFile f(::strPagerDir + strFilename);
  if(!f.open(IO_ReadOnly))
    return false;

  while(!f.atEnd()) {
    f.readLine(strTmp, 1024);
    strBuffer += strTmp;
  }
  f.close();
  slotReceiveContent(strBuffer);
  return true;
    
}
void CPager::slotUpdateContent()
{

  QString strQuery =
    m_pConfig->getCategoryTree()->getContentQuery(nNumItems);
  
  theServer->queueRequest(CWWWRequest::R_GETCONTENT, strQuery);

  if(m_pConfig->anyProfiles()) {
    if(bDebug) 
      cout << "Sending VULNPOLL" << endl;
    QString args;
    if(bVulnTest)
      args.sprintf("timestamp=0&version=%d", m_pConfig->getVulnVersion());
    else
      args.sprintf("timestamp=%s&version=%d", m_pConfig->getVulnTimestamp().ascii(), m_pConfig->getVulnVersion());
    
    theServer->queueRequest(CWWWRequest::R_VULNPOLL, args);
  }
  // Reset timer
  m_pTimer->changeInterval(m_pConfig->getPollInterval() * 1000 * 60);
    
}
// strContentInfo contains a list of top level categories that have new content
void CPager::clearContentInfo()
{
  strContentInfo.truncate(0);
}
void CPager::addNewContentInfo(const QString &strNew)
{
  if(strContentInfo.contains(strNew))
    return;
  if(!strContentInfo.isEmpty())
    strContentInfo += ", ";
  
  strContentInfo += strNew;

}
void CPager::slotReceiveContent(QString &strContent)
{
  QString strTopNews;
  QString strHeadline;

  QString strCategory;
  //  int nCategory; // XXX not used right now

  QString strLink;
  QString strTitle;
  QString strID;
  QString strTimestamp;
  QString strTopCategory;
  QString strNewFlag;
  int nID, nTopCategory;
  bool bOK, bNew;

  bAnythingNew = false;
  clearContentInfo();


  CWWWServer::xmlExtract("topnews", strContent, strTopNews);

  if(strTopNews.contains("<!-- error old category file")) {
    if(bDebug)
      cout << "old category file" << endl;
    theServer->queueRequest(CWWWRequest::R_UPDATECATEGORY, NULL);
    return;
  }
    
  while(CWWWServer::xmlExtract("headline", strTopNews, strHeadline)) {
    bNew = true;
    if(!CWWWServer::xmlExtract("category", strHeadline, strCategory))     
      continue;
    if(!CWWWServer::xmlExtract("link", strHeadline, strLink))      
      continue;
    if(!CWWWServer::xmlExtract("title", strHeadline, strTitle))      
      continue;
    if(!CWWWServer::xmlExtract("id", strHeadline, strID))      
      continue;
    if(!CWWWServer::xmlExtract("timestamp", strHeadline, strTimestamp))      
      continue;
    if(!CWWWServer::xmlExtract("topcategory", strHeadline, strTopCategory))      
      continue;

    if(CWWWServer::xmlExtract("newflag", strHeadline, strNewFlag) == true) 
      if(strNewFlag.toInt() == 0)
	bNew = false;

    nID = strID.toInt(&bOK);
    if(!bOK)
      continue;

    nTopCategory = strTopCategory.toInt(&bOK);
    if(!bOK)
      continue;
    slotNewContentItem(strTitle, strLink, strTimestamp, nID, nTopCategory, bNew);
  }

  if(bAnythingNew && bInit)
    slotNotifyContent();

}

void CPager::slotNotifyContent()
{
  QString strMessage;
  if(bDebug)
    cout << "New Content" << endl;
  if(m_pConfig->getFlashIcon())
    startFlashIcon();

  if(m_pConfig->getPopup()) {
    m_pTimer->stop();
    strMessage.sprintf("There is new SecurityFocus Pager content in the following categories:\n\n%s", strContentInfo.ascii());
    QMessageBox::information(0, "SecurityFocus Pager", strMessage, QMessageBox::Ok | QMessageBox::Default);
    stopFlashIcon();
    m_pTimer->changeInterval(m_pConfig->getPollInterval() * 1000 * 60);
  }
}
void CPager::slotNewContentItem(QString &strTitle, QString &strLink, QString &strTimestamp, int nID, int nTopCategory, bool bNew)
{

  CCategoryItem *topitem;
  CArticleItem *item;
  bool bStaleItem = false;

  topitem = (CCategoryItem *)m_pContentTree->firstChild();
  while(topitem) {
    if(topitem->getNodeID() == nTopCategory) {

      item = (CArticleItem *)topitem->firstChild();
      while(item) {
	if(item->getID() == nID) // item already exists
	  return;
	
	item = (CArticleItem *)item->nextSibling();
      }

      // Everything seems ok, so lets add it

      item =  new CArticleItem(topitem, strTitle, strLink, strTimestamp, nID, nTopCategory);

      if(bNew == false)
	item->setNew(false);
      
      m_pContentTree->repaintItem(topitem);      

      // If too many
      while(topitem->childCount() > nNumItems) {
	// In some wacky cases the oldest item will be the item we just added, so we don't want to set the bAnythingNew flag
	if(item == oldestArticle(topitem)) 
	  bStaleItem = true;
	CArticleItem *article = oldestArticle(topitem);
	if(article)
	  article->setNew(false);
	delete (article);
      }

      if(!bStaleItem) {
	addNewContentInfo(topitem->text(0));
	bAnythingNew = true;
      }

      return;
      
    }
    //
    // I added this again here in case the user reduces the number of article items and then immediately updates,
    // without this the number of items will not be reduced until there is new content
    //
    while(topitem->childCount() > nNumItems) {
      CArticleItem *article = oldestArticle(topitem);
      if(article)
	article->setNew(false);
      delete article;
    }
    topitem = (CCategoryItem *)topitem->nextSibling();
  }
}

CArticleItem *CPager::oldestArticle(CCategoryItem *top)
{
  CArticleItem *item = (CArticleItem *)top->firstChild();
  CArticleItem *oldest = item;

  while(item) {
    if(item->getTimestamp() < oldest->getTimestamp())
      oldest = item;

    item = (CArticleItem *) item->nextSibling();
  }

  return (oldest);
}

void CPager::slotReceiveAbstract(QString &text)
{

  QString strAbstract;
  CWWWServer::xmlExtract("body", text, strAbstract);
  CWWWServer::removeTags(strAbstract);
  m_pAbstract->setText(strAbstract);
}
void CPager::slotArticleSelected(QListViewItem *item)
{

  // This occasionally gets called with a NULL item
  if(!item) 
    return;
  CGenericItem *generic = (CGenericItem *)item;
  generic->doSelectedStuff(theServer);
  return;

}

void CPager::slotArticleDoubleClicked(QListViewItem *item)
{
  if(!item)
    return;
  CGenericItem *generic = (CGenericItem *)item;
  generic->doDoubleClickedStuff();
  return;
}

Atom XA_MOZILLA_VERSION = 0;
bool CPager::isNetscapeActive()
{
  // thanks JWZ
  if(!XA_MOZILLA_VERSION)
    XA_MOZILLA_VERSION = XInternAtom(qt_xdisplay(), "_MOZILLA_VERSION", False);

  Window root, parent, *kids;
  unsigned int nkids;
  XSync(qt_xdisplay(), false);
  if(XQueryTree(qt_xdisplay(), qt_xrootwin(), &root, &parent, &kids, &nkids)) 
    if(kids && nkids) {
      for(int i = nkids - 1; i >= 0; i--) {
	Atom type;
	int format;
	unsigned long nitems, nbytesafter;
	unsigned char *version = 0;
	Window w = XmuClientWindow(qt_xdisplay(), kids[i]);
	int status = XGetWindowProperty(qt_xdisplay(), w, XA_MOZILLA_VERSION, 0,
					(65536 / sizeof(long)), False, XA_STRING, &type, &format, &nitems,
					&nbytesafter, &version);
	if(!version)
	  continue;
	if(status == Success && type != None) {
	  XFree(version);
	  XFree(kids);
	  return true;
	}
	XFree(version);
      }
      XFree(kids);
    }
  return false;
}


void CPager::SurfItUp(const QString &strURL, bool bRemote)
{

  int pid;
  QString strTmp;
  bool bNetscapeActive = false;

  if(::strBrowser.isEmpty()) {
    QMessageBox::warning(0, "No Browser Specified", "You must configure the path to your web browser to navigate to content");
    return;
  }
  QFileInfo fi(::strBrowser);
  if(! (fi.exists() && fi.isFile() && fi.isExecutable())){
    QMessageBox::warning(0, "Bad Browser", "The browser that you have configured is not a valid executable");
    return;
  }
  
    
  if(bDebug)
    cout << "navigating to: " << strURL << endl;
  if(::strBrowser.contains("netscape") && bRemote)
     bNetscapeActive = isNetscapeActive();

  // fork
  if( (pid = fork()) > 0) 
    return;

  if(pid == -1) {
    qWarning("Fork to launch browser failed");
    return;
  }

  // exec
       
  if(bNetscapeActive) {
    // Netscape -remote
    strTmp.sprintf("openURL(%s)", strURL.ascii());
    execl(::strBrowser.ascii(), ::strBrowser.ascii(), "-remote", strTmp.ascii(), NULL);
    qWarning("Netscape was detected to be active on your desktop but was unable to navigate to selected content.");
    exit(0);
  }

  execl(::strBrowser.ascii(), ::strBrowser.ascii(), strURL.ascii(), NULL);
  qWarning("Could not launch the browser that you specified in configuration.");

  exit(0);
}

void CPager::slotDoConfig()
{

  // Disable update timer
  m_pTimer->stop();

  int ret =  m_pConfig->exec();
  if(ret == QDialog::Accepted) {
    m_pConfig->getCategoryTree()->saveCategories(szCategoryFile);
    slotUpdateTopLevel();

    if(nPollInterval != m_pConfig->getPollInterval()) {
      nPollInterval = m_pConfig->getPollInterval();
      m_pTimer->changeInterval(nPollInterval * 1000 * 60);
    }
    nNumItems = m_pConfig->getNumItems();

    if(m_pConfig->getConfigRestart())
      restart();

    // Switch from NULL skin to chosen skin
    if(currentSkin == NULL && (!m_pConfig->getSkinDirectory().isEmpty())) {
      currentSkin = new CSkin(m_pConfig->getSkinDirectory());
      currentSkin->apply();
    }
    // Switch to NULL skin if it has been selected
    else if(m_pConfig->getSkinDirectory().isEmpty()) { // "No Skin" has been selected
      if(currentSkin) {
	delete currentSkin;
	currentSkin = NULL;
	CSkin::applyNull();
	gPagerMain->resize(600,300);
      }   
    }
    // Switch from current skin to selected skin
    else if(currentSkin == NULL || currentSkin->getName() != m_pConfig->getChosenSkin()) {
      delete currentSkin;
      currentSkin = new CSkin(m_pConfig->getSkinDirectory());
      currentSkin->apply();
    }


    ::strBrowser = m_pConfig->getBrowserPath();
    slotUpdateContent();
  }
  m_pTimer->changeInterval(m_pConfig->getPollInterval() * 1000 * 60);
  
}

void CPager::quit()
{
  saveArticles(szArticleFile);
  saveConfig(szConfigFile);
  
}

void CPager::restart()
{
  saveArticles(szArticleFile);
  saveConfig(szConfigFile);
  
  if(!fork()) 
    execlp(qApp->argv()[0], qApp->argv()[0], NULL);
  exit(0);

}

bool CPager::loadConfig(const QString &strFilename)
{
  QString strBuffer, strTmp;
  QString strConfig, strNumItems, strPollInterval;
  QString strBrowserPath;
  QString strFlashIcon, strPopup;
  QString strRestart;
  QString strSkin;
  bool bFlashIcon, bPopup, bRestart;

  m_pConfig->setSkin("Max");

  QFile f(::strPagerDir + strFilename);
  if(!f.open(IO_ReadOnly))
    return false;

  while(!f.atEnd()) {
    f.readLine(strTmp, 1024);
    strBuffer += strTmp;
  }
  f.close();

  if(!CWWWServer::xmlExtract("config", strBuffer, strConfig))
    return false;
  if(!CWWWServer::xmlExtract("numitems", strConfig, strNumItems))
    return false;
  if(!CWWWServer::xmlExtract("pollinterval", strConfig, strPollInterval))
    return false;
  if(!CWWWServer::xmlExtract("browserpath", strConfig, strBrowserPath))
    return false;
  if(!CWWWServer::xmlExtract("flashicon", strConfig, strFlashIcon))
    return false;
  if(!CWWWServer::xmlExtract("popup", strConfig, strPopup))
    return false;
  if(!CWWWServer::xmlExtract("configrestart", strConfig, strRestart))
    return false;
  if(!CWWWServer::xmlExtract("skin", strConfig, strSkin))
    ;

  nNumItems = strNumItems.toInt();
  if(nNumItems < COptions::nMinItems)
    nNumItems = COptions::nMinItems;
  if(nNumItems > COptions::nMaxItems)
    nNumItems = COptions::nMaxItems;

  nPollInterval = strPollInterval.toInt();
  if(nPollInterval < COptions::nMinInterval)
    nPollInterval = COptions::nMinInterval;
  if(nPollInterval > COptions::nMaxInterval)
    nPollInterval = COptions::nMaxInterval;

  bFlashIcon = (strFlashIcon.toInt() == 1);
  bPopup = (strPopup.toInt() == 1);
  bRestart = (strRestart.toInt() == 1);

  m_pConfig->setOptionsStuff(nNumItems, nPollInterval, strBrowserPath, bFlashIcon, bPopup, bRestart);
  
  // Network Inventory Options stuff
  QString strPopupOn, strVerboseOn, strBriefOn, strVerboseEmail, strBriefEmail, strMailer;
  bool bVulnPopup, bVerbose, bBrief;
  if(!CWWWServer::xmlExtract("vulnpopup", strConfig, strPopupOn))
    return false;
  if(!CWWWServer::xmlExtract("verbosenotify", strConfig, strVerboseOn))
    return false;
  if(!CWWWServer::xmlExtract("briefnotify", strConfig, strBriefOn))
    return false;
  if(!CWWWServer::xmlExtract("verboseemail", strConfig, strVerboseEmail))
    return false;
  if(!CWWWServer::xmlExtract("briefemail", strConfig, strBriefEmail))
    return false;
  if(!CWWWServer::xmlExtract("mailer", strConfig, strMailer))
    return false;

  bVulnPopup = (strPopupOn.toInt() == 1);
  bVerbose = (strVerboseOn.toInt() == 1);
  bBrief = (strBriefOn.toInt() == 1);

  m_pConfig->setInventoryStuff(strVerboseEmail, strBriefEmail, strMailer, bVulnPopup, bVerbose, bBrief);
  if(!strSkin.isEmpty()) 
    m_pConfig->setSkin(strSkin);

  return true;
}

bool CPager::saveConfig(const QString &strFilename)
{
  QFile f(::strPagerDir + strFilename);
  
  if(!f.open(IO_WriteOnly | IO_Truncate))
    return false;

  QTextStream t(&f);
  t << "<config>" << endl;
  t << "\t<numitems>" << nNumItems << "</numitems>" << endl;
  t << "\t<pollinterval>" << nPollInterval << "</pollinterval>" << endl;
  t << "\t<browserpath>" << m_pConfig->getBrowserPath() << "</browserpath>" << endl;
  t << "\t<flashicon>" << (m_pConfig->getFlashIcon()?"1":"0") << "</flashicon>" << endl;
  t << "\t<popup>" << (m_pConfig->getPopup()?"1":"0") << "</popup>" << endl;
  t << "\t<skin>" << m_pConfig->getChosenSkin() << "</skin>" << endl;

  t << "\t<vulnpopup>" << (m_pConfig->getVulnPopup()?"1":"0") << "</vulnpopup>" << endl;
  t << "\t<verbosenotify>" << (m_pConfig->getVerboseNotify()?"1":"0") << "</verbosenotify>" << endl;
  t << "\t<briefnotify>" << (m_pConfig->getBriefNotify()?"1":"0") << "</briefnotify>" << endl;
  t << "\t<configrestart>" << (m_pConfig->getConfigRestart()?"1":"0") << "</configrestart>" << endl;
  t << "\t<verboseemail>" << m_pConfig->getVerboseEmail() << "</verboseemail>" << endl;
  t << "\t<briefemail>" << m_pConfig->getBriefEmail() << "</briefemail>" << endl;
  t << "\t<mailer>" << m_pConfig->getMailer() << "</mailer>" << endl;
  t << "</config>" << endl;
  f.close();
  return true;
}

void CPager::slotReceiveAd(QString &strGif, QString &strLink)
{
  if(bDebug)
    cout << "slotReceiveAd: " << strGif << endl;
  theServer->queueRequest(CWWWRequest::R_FETCHADIMAGE, strGif);
  strBannerLink = strLink;
  m_pBannerTimer->changeInterval(120 * 1000);

}
void CPager::slotBannerNavigate()
{
  CPager::SurfItUp(strBannerLink, true);
}
void CPager::slotReceiveVulns()
{
  if(bDebug)
    cout << "slotReceiveVulns" << endl;

  m_pConfig->processVulns();
  return;
}

void CPager::slotFlashTimeout()
{
  bIconOne = !bIconOne;
  emit newIcon(bIconOne);
}

void CPager::startFlashIcon()
{
  bIconOne = true;
  m_pFlashTimer->changeInterval(300);
}

void CPager::stopFlashIcon()
{
  m_pFlashTimer->stop();
  emit newIcon(true);
}

void CPager::slotGetAd()
{
  // Only request an ad from server if pager is focused
  if(bFocus)
    theServer->queueRequest(CWWWRequest::R_FETCHAD, NULL);
}

void CPager::doHelp()
{
  SurfItUp(szHelpURL, true);
}

void CPager::slotDoSearch()
{
  CSearchDialog d;
  QString strSearch;
  QString strRequest;
  int n = 0;

  if(d.exec()) {
    strSearch = d.getSearch().simplifyWhiteSpace();
    while( (n = strSearch.find(' ', n)) != -1) 
      strSearch.replace(n, 1, "+");
    strRequest.sprintf("http://www.securityfocus.com/search/index.html?base=security_focus&p=%s&start=0", strSearch.ascii());
    CPager::SurfItUp(strRequest, false);
  }
    
}

