#ifndef PAGER_H
#define PAGER_H

#include <qapplication.h>
#include <qvbox.h>
#include <qpixmap.h>
#include <qpoint.h>
#include <qtimer.h>
#include <iostream.h>
#include <qevent.h>
#include <qtooltip.h>
#include <qpushbutton.h>
#include "wwwserver.h"
#include "pagerbutton.h"
#include "banner.h"
#include "wizard.h"
#include "skin.h"
#include "resources/icon1.xpm"
#include "resources/icon2.xpm"
class CArticleItem;
class CConfig;
class CCategoryItem;
class QSplitter;
class QListView;
class QListViewItem;
class QTextBrowser;
extern CMain *gPagerMain;
extern QString strPagerDir;

class CPager : public QVBox {
  Q_OBJECT
 public:
  static const char szServerName[] = "www.securityfocus.com";
  static const char szHelpURL[] = "file:/usr/lib/SecurityFocus/pagerdoc/pager.html";
  static const char szConfigFile[] = "sfpager.cfg";
  static const char szCategoryFile[] = "category.dat";
  static const char szArticleFile[] = "article.dat";
  static const char szEmptyAbstractText[] = "<center>SecurityFocus Pager</center>"
    "<center>Version 3.0</center>" ;


  static QString strPagerDir;
  static QString strBrowser;
  static bool isNetscapeActive();
  static void SurfItUp(const QString &strURL, bool bRemote);
    
  CPager(QWidget *parent = 0, const char *name = 0);
  void applySkin();
  bool loadArticles(const QString &strFilename);
  bool saveArticles(const QString &strFilename);
  CWWWServer *theServer;  
  void isFocus(bool b) { bFocus = b; }
  void startFlashIcon();
  void stopFlashIcon();
  bool isNullSkin() { return (currentSkin == NULL); }
 protected:

  CWizard wizard;
  CSkin *currentSkin;
  CPagerButton *m_pConfigButton;
  QPushButton *m_pUpdateButton;
  QPushButton *m_pQuitButton;
  QSplitter *m_pSplitter;
  QListView *m_pContentTree;
  QTextBrowser *m_pAbstract;
  CConfig *m_pConfig;
  QString strBannerLink;

  QTimer *m_pTimer;
  QTimer *m_pBannerTimer;
  QTimer *m_pFlashTimer;
  int nVersion;
  int nNumItems;
  int nPollInterval;
  bool  bInit;
  bool bFocus;
  bool bAnythingNew;
  bool bIconOne;
  CArticleItem *oldestArticle(CCategoryItem *top);
  bool loadConfig(const QString &strFilename);
  bool saveConfig(const QString &strFilename);
  bool setupDirectory();
  void clearContentInfo();
  void addNewContentInfo(const QString &strNew);
  QString strContentInfo;
  public slots:
    void slotUpdateTopLevel();
    void slotUpdateContent();
    void slotReceiveContent(QString &strContent);
    void slotReceiveAbstract(QString &abstract);
    void slotArticleSelected(QListViewItem *item);
    void slotArticleDoubleClicked(QListViewItem *item);
    void slotNewContentItem(QString &strTitle, QString &strLink, QString &strTimestamp, int nID, int nTopCategory, bool bNew);
    void slotDoConfig();
    void slotDoSearch();
    void slotReceiveVulns();
    void slotReceiveAd(QString &strGif, QString &strLink);
    void slotBannerNavigate();
    void slotNotifyContent();
    void quit();
    void restart();
    void slotFlashTimeout();
    void slotGetAd();
    void doHelp();
    char *resolvServer(const char *hostname);
 signals:
    void newContentItem(QString &strTitle, QString &strLink, QString &strTimestamp, int nID, int nTopCategory);
    void newIcon(bool bOne);
  
};


class CMain : public QWidget
{
Q_OBJECT
 public:
 CPager *thePager;
 CPagerButton *config_button, *search_button, *update_button, *min_button, *exit_button, *help_button; 
 QPushButton *config_plain, *search_plain, *update_plain, *help_plain;
 QHBox *plain_buttons;
 CBanner *ad;
 CMain();
protected:
 void mousePressEvent(QMouseEvent *event) { grabMouse(QCursor(pointingHandCursor));last = event->pos(); raise(); thePager->stopFlashIcon();}
  void mouseMoveEvent(QMouseEvent *event) { 
    move(event->globalPos() - last);
  }
  void mouseReleaseEvent(QMouseEvent *) { releaseMouse(); }
  void focusInEvent(QFocusEvent *) { thePager->isFocus(true);}
  void focusOutEvent(QFocusEvent *) { thePager->isFocus(false); }
  void hideEvent(QHideEvent *) { thePager->isFocus(false); }
  void showEvent(QShowEvent *) { thePager->isFocus(true); thePager->stopFlashIcon(); }
  void resizeEvent(QResizeEvent *e) {
    if(!bInit) return;
    if(!thePager->isNullSkin())
      return;
    thePager->setGeometry(0, 50, e->size().width(), e->size().height() - 95);
    plain_buttons->setGeometry(0, 0, e->size().width(), 50);
    ad->setGeometry(e->size().width() - 235, e->size().height() - 38, 230, 33);
  }
  QPoint last;
  QLabel *logo;
  bool bInit ;
  bool eventFilter(QObject *object, QEvent *event);

 
protected slots:
  void slotChangeIcon(bool bOne) {
  if(bOne)
    setIcon(icon1_xpm);
  else
    setIcon(icon2_xpm);
}
};
#endif // PAGER_H


























