#include <qfile.h>
#include <qbitmap.h>
#include <qpushbutton.h>
#include "skin.h"
#include "pager.h"
#include "pagerbutton.h"
#include "banner.h"
#include <X11/Xlib.h>
#include <iostream>
typedef struct
{
  unsigned long flags;
  unsigned long functions;
  unsigned long decorations;
  long inputMode;
  unsigned long status;
} PropMotifWmHints;
// Stuff to remove window decorations
#define MWM_HINTS_FUNCTIONS (1L << 0)
#define MWM_HINTS_DECORATIONS (1L << 1)
#define MWM_HINTS_INPUT_MODE (1L << 2)
#define MWM_HINTS_STATUS (1L << 3)
#define MWM_DECOR_ALL (1L << 0)
#define MWM_DECOR_BORDER (1L << 1)
#define MWM_DECOR_RESIZEH (1L << 2)
#define MWM_DECOR_TITLE (1L << 3)
#define MWM_DECOR_MENU (1L << 4)
#define MWM_DECOR_MINIMIZE (1L << 5)
#define MWM_DECOR_MAXIMIZE (1L << 6)
#define PROP_MOTIF_WM_HINTS_ELEMENTS 5

void set_mwm_border(Display *dpy, Window w, unsigned long flags)
{
  PropMotifWmHints motif_hints;
  Atom prop, proptype;

  // setup property
  motif_hints.flags = MWM_HINTS_DECORATIONS;
  motif_hints.decorations = flags;

  // get the atom for the property
  prop = XInternAtom(dpy, "_MOTIF_WM_HINTS", true);
  if(!prop) 
    return;

  proptype = prop;
  XChangeProperty(dpy, w, prop, proptype, 32, PropModeReplace, (unsigned char *)&motif_hints, PROP_MOTIF_WM_HINTS_ELEMENTS);
}

extern bool bDebug;
extern CMain *gPagerMain;

CSkin::CSkin(const QString &strDir)
  : info_main(0, 0, 0, 0), info_logo(0, 0), info_banner(0, 0), info_min(0, 0), 
  info_exit(0, 0), info_help(0, 0), info_search(0, 0), info_update(0, 0), info_config(0, 0)
{
  QPoint mainloc;
  QString strLine;
  int mainwidth = 0, mainheight = 0;
  bInitialized = false;
  setupFiles();
  QFile f(strDir + "/skin.ini");
  strPath = strDir;
  
  if(!f.open(IO_ReadOnly)) {
    if(bDebug)
      cout << "Cannot open " << strPath << "/skin.ini" << endl;
    return;
  }

  while(!f.atEnd()) {
    if( (f.readLine(strLine, 1024) == -1) && (bDebug))
      cout << "error reading from skin file" << endl;
    strLine = strLine.simplifyWhiteSpace();
    if(strLine.isEmpty())
      continue;
    if( *strLine.ascii() == ';')
      continue;
    if( (strLine.find("[skin]", 0, false) == 0) && (strLine.length() == 6) )
      continue;

    if((strLine.find("name=", 0, false) == 0)) {
      strName = strLine.mid(5);
      strName = strName.stripWhiteSpace();
    }
    else if((strLine.find("main=", 0, false) == 0))
      mainloc = pair2point(strLine.mid(5));
    else if((strLine.find("mainwidth=", 0, false) == 0))
      mainwidth = strLine.mid(10).toInt();
    else if((strLine.find("mainheight=", 0, false) == 0))
      mainheight = strLine.mid(11).toInt();
    else if((strLine.find("logo=", 0, false) == 0))
      info_logo = pair2point(strLine.mid(5));
    else if((strLine.find("banner=", 0, false) == 0))
      info_banner = pair2point(strLine.mid(7));
    else if((strLine.find("help=", 0, false) == 0))
      info_help = pair2point(strLine.mid(5));
    else if((strLine.find("min=", 0, false) == 0))
      info_min = pair2point(strLine.mid(4));
    else if((strLine.find("exit=", 0, false) == 0))
      info_exit = pair2point(strLine.mid(5));
    else if((strLine.find("search=", 0, false) == 0))
      info_search = pair2point(strLine.mid(7));
    else if((strLine.find("update=", 0, false) == 0))
      info_update = pair2point(strLine.mid(7));
    else if((strLine.find("config=", 0, false) == 0))
      info_config = pair2point(strLine.mid(7));
    else if(bDebug)
      cout << "Could not parse line from skin configuration file" << endl;
  }
    
  if(strName.isEmpty() || mainloc.isNull() || mainwidth == 0 || mainheight == 0 || info_banner.isNull() 
     || info_help.isNull() || info_min.isNull() || info_exit.isNull() || info_search.isNull() || info_update.isNull() || info_config.isNull())
    // XXX should set all these variables to null again
    return;

  if(bDebug) 
    cout << "Skin initialized successfully" << endl;

  info_main.setRect(mainloc.x(), mainloc.y(), mainwidth, mainheight);
  bInitialized = true;
    
}

QPoint CSkin::pair2point(const QString &strPair) const
{
  int n;
  if( (n = strPair.find(",", 0)) == -1)
    return QPoint();

  return QPoint(strPair.left(n).toInt(), strPair.mid(n + 1).toInt());

}
void CSkin::setupFiles()
{
  strFiles[BACKGROUND]   = "pager.bmp";
  strFiles[MASK]         = "pager-mask.bmp";
  strFiles[HELP]         = "help.bmp";
  strFiles[HELPCLICK]    = "helpclick.bmp";
  strFiles[MIN]          = "min.bmp";
  strFiles[MINCLICK]     = "minclick.bmp";
  strFiles[EXIT]         = "exit.bmp";
  strFiles[EXITCLICK]    = "exitclick.bmp";
  strFiles[CONFIG]       = "config.bmp";
  strFiles[CONFIGCLICK]  = "configclick.bmp";
  strFiles[SEARCH]       = "search.bmp";
  strFiles[SEARCHCLICK]  = "searchclick.bmp";
  strFiles[UPDATE]       = "update.bmp";
  strFiles[UPDATECLICK]  = "updateclick.bmp";
}

// 
// Apply this skin to pager
//
//bool CSkin::apply(CMain *pMain)
bool CSkin::apply()
{
  CMain *pMain = gPagerMain;

  if(!bInitialized)
    return false;

  for(GRAPHIC g = BACKGROUND; g < MAXGRAPHIC; g = (GRAPHIC)(g + 1)) {
    Pixmaps[g] = new QPixmap(strPath + "/" + strFiles[g]);
    if(Pixmaps[g]->isNull()) {
      if(bDebug)
	cout << "Skin Load Failed: uh oh, could not load " << strFiles[g] << endl;
      exit(1);
    }
  }

  pMain->plain_buttons->hide();
  QBitmap bm;
  bm = *(Pixmaps[MASK]);
  pMain->setMask(bm);
  pMain->setBackgroundPixmap(*Pixmaps[BACKGROUND]);
  pMain->resize(Pixmaps[BACKGROUND]->size().width(), Pixmaps[BACKGROUND]->size().height());
  // Set geometry of main window
  pMain->thePager->setGeometry(info_main);

  pMain->config_button->setPixmaps(Pixmaps[CONFIG], Pixmaps[CONFIGCLICK]);  
  pMain->config_button->setGeometry(info_config.x(), info_config.y(), Pixmaps[CONFIG]->size().width(), Pixmaps[CONFIG]->size().height());
  pMain->config_button->show();

  pMain->update_button->setPixmaps(Pixmaps[UPDATE], Pixmaps[UPDATECLICK]);
  pMain->update_button->setGeometry(info_update.x(), info_update.y(), Pixmaps[UPDATE]->size().width(), Pixmaps[UPDATE]->size().height());
  pMain->update_button->show();

  pMain->search_button->setPixmaps(Pixmaps[SEARCH], Pixmaps[SEARCHCLICK]);
  pMain->search_button->setGeometry(info_search.x(),info_search.y(), Pixmaps[SEARCH]->size().width(), Pixmaps[SEARCH]->size().height());
  pMain->search_button->show();

  pMain->min_button->setPixmaps(Pixmaps[MIN], Pixmaps[MINCLICK]);
  pMain->min_button->setGeometry(info_min.x(), info_min.y(), Pixmaps[MIN]->size().width(), Pixmaps[MIN]->size().height());
  pMain->min_button->show();

  pMain->exit_button->setPixmaps(Pixmaps[EXIT], Pixmaps[EXITCLICK]);
  pMain->exit_button->setGeometry(info_exit.x(), info_exit.y(), Pixmaps[EXIT]->size().width(), Pixmaps[EXIT]->size().height());
  pMain->exit_button->show();

  pMain->help_button->setPixmaps(Pixmaps[HELP], Pixmaps[HELPCLICK]);
  pMain->help_button->setGeometry(info_help.x(), info_help.y(), Pixmaps[HELP]->size().width(), Pixmaps[HELP]->size().height());
  pMain->help_button->show();
  
  pMain->ad->setGeometry(info_banner.x(), info_banner.y(), 230, 33);
  pMain->ad->slotChangeBanner();
  pMain->ad->installEventFilter(pMain);
  QObject::connect(pMain->thePager->theServer, SIGNAL(newAdImage()), pMain->ad, SLOT(slotChangeBanner()));
  QObject::connect(pMain->ad, SIGNAL(bannerClicked()), pMain->thePager, SLOT(slotBannerNavigate()));

  set_mwm_border(qt_xdisplay(), pMain->winId(), 0);
  // pMain->hide(); pMain->show();
  return true;
    
}
void CSkin::applyNull()
{
  CMain *pMain = gPagerMain;

  pMain->config_button->hide();
  pMain->update_button->hide();
  pMain->search_button->hide();
  pMain->min_button->hide(); 
  pMain->help_button->hide();
  pMain->exit_button->hide();
  pMain->plain_buttons->show();


  pMain->clearMask();
  pMain->setBackgroundMode(QWidget::PaletteBackground);
  pMain->ad->slotChangeBanner();
  pMain->ad->installEventFilter(pMain);
  QObject::connect(pMain->thePager->theServer, SIGNAL(newAdImage()), pMain->ad, SLOT(slotChangeBanner()));
  QObject::connect(pMain->ad, SIGNAL(bannerClicked()), pMain->thePager, SLOT(slotBannerNavigate()));
  
  set_mwm_border(qt_xdisplay(), pMain->winId(), MWM_DECOR_ALL);

  //  pMain->hide(); pMain->show();

}
