#include "wizard.h"
#include <qwidget.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qfiledialog.h>
#include <qpushbutton.h>
#include <qtextbrowser.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qstring.h>
#include <qfileinfo.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <iostream.h>

CWizard::CWizard(QWidget *parent, const char *name)
  : QWizard(parent, name, TRUE)
{
  resize(700,350);
  setupPage0();
  setupPage1();
  setupPage2();
  setupPage3();
  setupPage4();
}

void CWizard::setupPage0()
{
  page0 = new QVBox(this);
  QTextView *tv = new QTextBrowser(page0);
  QString licText, strBuf;
  int i;

  QFile f("/usr/lib/SecurityFocus/LICENSE");
  if(!f.open(IO_ReadOnly)) {
    cerr << "Could not open LICENSE file" << endl;
    exit(0);
  }
  while(!f.atEnd()) {
    f.readLine(strBuf, 1024);
    licText += strBuf;
  }
  f.close();
  while((i = licText.find("\n", 0, false)) != -1)
    licText.replace(i, 1, "<br>");
  tv->setText(licText);
  tv->setTextFormat(RichText);    
  nextButton()->setText("I Accept");
  cancelButton()->setText("I DO NOT Accept");
  helpButton()->hide();
  backButton()->hide();
  setNextEnabled(page1, true);
  
  addPage(page0, "License");
  
}
void CWizard::setupPage1()
{
  page1 = new QVBox(this);

  QLabel *l = new QLabel(page1);
  l->setText("Welcome to the SecurityFocus Pager application.\n\n"
	     "The next few dialogs will help you set up the Pager for\n"
	     "for your system configuration.\n\n"
	     "Click on the Next button to get started.\n");

  addPage(page1, "Welcome");

  setBackEnabled(page1, false);
  setNextEnabled(page1, true);
  setHelpEnabled(page1, false);
}

void CWizard::setupPage2()
{

  QFileInfo f(szDefaultBrowserPath);
  
  page2 = new QVBox(this);
  QLabel *l = new QLabel(page2);
  l->setText("In order to browse content on the SecurityFocus web site,\n"
	     "the Pager needs to know where to find your preferred\n"
	     "web browser.  Enter the full path below.\n\n");

  QHBox *h = new QHBox(page2);

  if(f.exists() && f.isFile() && f.isExecutable())
    browserPath = new QLineEdit(szDefaultBrowserPath, h);
  else
    browserPath = new QLineEdit(h);

  QObject::connect(browserPath, SIGNAL(textChanged(const QString &)), SLOT(dataChanged(const QString &)));
  QPushButton *p = new QPushButton("Browse", h);
  QObject::connect(p, SIGNAL(clicked()), SLOT(doBrowse()));

  addPage(page2, "Browser Path");
  setNextEnabled(page2, (!browserPath->text().isEmpty()));
  setHelpEnabled(page2, false);
}

void CWizard::setupPage3()
{
  QString strMailer = szDefaultMailerCommand;
  if(strMailer.find(' ') != -1)
    strMailer.truncate(strMailer.find(' '));

  QFileInfo f(strMailer);
  page3 = new QVBox(this);
  QLabel *l = new QLabel(page3);
  l->setText("The network inventory feature of the pager includes the ability\n"
	     "to send email notification of vulnerabilities.  To use this \n"
	     "feature the pager needs to know where to find a mail program\n"
	     "on your system.  Please enter the full path below.\n\n");
  QHBox *h = new QHBox(page3);

  if(f.exists() && f.isFile() && f.isExecutable())
    mailerPath = new QLineEdit(szDefaultMailerCommand, h);
  else
    mailerPath = new QLineEdit(h);
  QObject::connect(mailerPath, SIGNAL(textChanged(const QString &)), SLOT(dataChanged(const QString &)));
  QPushButton *p = new QPushButton("Browse", h);
  QObject::connect(p, SIGNAL(clicked()), SLOT(doBrowse()));

  addPage(page3, "Mailer Path");
  setNextEnabled(page3, (!mailerPath->text().isEmpty()));
  setHelpEnabled(page3, false);
}

void CWizard::setupPage4()
{
  page4 = new QVBox(this);
  QLabel *l = new QLabel(page4);
  l->setText("You are now finished configuring the SecurityFocus Pager.\n"
	     "Click the Finish button to launch the Pager.\n");
  addPage(page4, "Finished");
  setFinish(page4, true);
  setHelpEnabled(page4, false);
}

void CWizard::showPage(QWidget *page)
{
  QString strPath;
  

  if(page == page1) {
    nextButton()->setText("Next >");
    cancelButton()->setText("&Cancel");
    backButton()->show();
  }
  if(page == page3) {
    strPath = browserPath->text();
    QFileInfo f(strPath);
    if(!(f.exists() && f.isFile() && f.isExecutable())) {
      QMessageBox::information(this, "SecurityFocus Pager Wizard",  strPath + " is not a path to a valid executable file");
      return;
    }
  }

  if(page == page4) {
    strPath = mailerPath->text();
    if(strPath.find(' ') != -1)
      strPath.truncate(strPath.find(' '));
    QFileInfo f(strPath);
    if(!(f.exists() && f.isFile() && f.isExecutable())) {
      QMessageBox::information(this, "SecurityFocus Pager Wizard", strPath + " is not a path to a valid executable file");
      return;
    }
  }

  QWizard::showPage(page);

  if(page == page4) {
    finishButton()->setEnabled(true);
    finishButton()->setFocus();
  }
}

void CWizard::dataChanged(const QString &strNew)
{
  if(strNew.isEmpty())
    nextButton()->setEnabled(false);
  else
    nextButton()->setEnabled(true);
}

void CWizard::doBrowse()
{
  QString strFile = QFileDialog::getOpenFileName("/");
  
  if(currentPage() == page2)
    browserPath->setText(strFile);
  else if(currentPage() == page3)
    mailerPath->setText(strFile);

}

