#include "wwwserver.h"
#include "pager.h"
#include <qstring.h>
#include <qfile.h>
#include <qsocket.h>
#include <qtextstream.h>
#include <iostream>


extern bool bDebug;

CWWWServer::CWWWServer(const QString strServer)
  :  strWWWServerName(strServer), strWWWUpdateURL(szUpdateURL), strWWWUpdateVulnURL(szUpdateVulnURL),
     bBusy(false), state(ST_IDLE)
{
  s = new QSocket;
  requestQueue.setAutoDelete(true);
  QObject::connect(s, SIGNAL(connected()), this, SLOT(slotDoConnected()));
  QObject::connect(s, SIGNAL(readyRead()), this, SLOT(slotDoRead()));
  QObject::connect(s, SIGNAL(closed()), this, SLOT(slotDoClose()));
}

CWWWServer::~CWWWServer() {
  s->close();
  delete s;
}

void CWWWServer::queueRequest(CWWWRequest::Request request, const QString &strArgument)
{
  requestQueue.enqueue(new CWWWRequest(request, strArgument));
  if(state == ST_IDLE)
    runQueue();
}

//
// If there are multiple requests for abstracts in the queue, only the most recent one should
// be recognized and it should be moved to the front of the queue.  That's what's going on here
//
void CWWWServer::cleanQueue()
{
  QQueue <CWWWRequest> abstractQueue;
  QQueue <CWWWRequest> otherQueue;
  CWWWRequest *requestItem;

  while(!requestQueue.isEmpty()) {
    requestItem = requestQueue.dequeue();
    if(requestItem->type == CWWWRequest::R_FETCHABSTRACT)
      abstractQueue.enqueue(requestItem);
    else
      otherQueue.enqueue(requestItem);
  }
  //
  // Ok, now abstractQueue has all of the outstanding requests for abstracts and the
  // otherQueue has the rest of the queued requests.  What we want to do is place the most
  // recent abstract request at the front of the new queue then everything from otherQueue
  // after
  //
  
  if(abstractQueue.isEmpty()) {
    requestQueue = otherQueue;
    return;
  }
  while(abstractQueue.count() > 1)
    abstractQueue.remove();
  
  requestQueue = abstractQueue;
  
  while(!otherQueue.isEmpty()) 
    requestQueue.enqueue(otherQueue.dequeue());
}

void CWWWServer::goIdle()
{

  if(!runQueue()) {
    state = ST_IDLE;
  }
}

bool CWWWServer::runQueue()
{
  cleanQueue();
  CWWWRequest *req = requestQueue.dequeue();
  if(!req)
    return false;

  strBuffer.truncate(0);

  
  switch(req->type) {
  case CWWWRequest::R_UPDATECATEGORY:
    if(bDebug)
      cout << "Sending category update" << endl;
    state = ST_UPDATECATEGORY;
    strQuery.sprintf("GET %s HTTP/1.0", strWWWUpdateURL.ascii());
    break;
  case CWWWRequest::R_UPDATEVULN:
    if(bDebug)
      cout << "Sending vuln update" << endl;
    state = ST_UPDATEVULN;
    strQuery.sprintf("GET %s HTTP/1.0", strWWWUpdateVulnURL.ascii());
    break;
  case CWWWRequest::R_FETCHABSTRACT:
    if(bDebug)
      cout << "Sending abstract fetch, arg = " << req->arg << endl;
    state = ST_FETCHABSTRACT;
    strQuery.sprintf("GET /pagerv3/abstract.html?%s HTTP/1.0", req->arg.ascii());
    break;
  case CWWWRequest::R_GETCONTENT:
    if(bDebug)
      cout << "Sending content request" << endl;
    state = ST_GETCONTENT;
    strQuery.sprintf("GET /pagerv3/pager_info.html?%s HTTP/1.0", req->arg.ascii());
    break;
  case CWWWRequest::R_VULNPOLL:
    state = ST_VULNPOLL;
    if(bDebug)
      cout << "vuln polling arg = " << req->arg << endl;
    strQuery.sprintf("GET /pagerv3/vuln_info.html?%s HTTP/1.0", req->arg.ascii());
    break;
  case CWWWRequest::R_FETCHAD:
    state = ST_FETCHAD;
    strQuery.sprintf("GET /frames/ad.html?group=pager HTTP/1.0");
    break;
  case CWWWRequest::R_FETCHADIMAGE:
    state = ST_FETCHADIMAGE;
    strQuery.sprintf("GET %s HTTP/1.0", req->arg.ascii());
    break;
  default:
    delete req;
    // XXX should print a warning maybe
    if(requestQueue.head()) 
      return runQueue();
    return false;
  }
  delete req;
  s->connectToHost(strWWWServerName, nServerPort);
  QApplication::setOverrideCursor(waitCursor);
  return true;
}

void CWWWServer::slotDoConnected()
{
  QString strSend = strQuery + "\r\n" +  "User-Agent: SFPagerV3.0(Linux)\r\n\r\n";
  if(state == ST_FETCHADIMAGE) {
    adfile = new QFile(strPagerDir + "ad.gif");
    if(!adfile->open(IO_Truncate | IO_WriteOnly)) {
      s->close();
      delete adfile;
      return;
    }
    s->setMode(QSocket::Binary);
    bAdGotHeader = false;
  }
  else
    s->setMode(QSocket::Ascii);
  s->writeBlock(strSend.ascii(), strSend.length());
}

void CWWWServer::slotDoRead()
{
  int i;
  char buffer[1025];
  char *p;
  if(state == ST_FETCHADIMAGE) {
    while(s->bytesAvailable()) {
      i = s->readBlock(buffer, 1024);
      buffer[i] = 0;
      p = buffer;
      // If we haven't read the header yet look for a blank line
      if(!bAdGotHeader) {
	if( (p = strstr(buffer, "\r\nGIF"))) {
	   bAdGotHeader = true;
	   p += 2;
	   i -= (p - buffer);
	} else if(strstr(buffer, "GIF") == buffer) {
	  bAdGotHeader = true;
	  p = buffer;
	}
      }
      if(bAdGotHeader) 
	adfile->writeBlock(p, i);
    }
	   
  }
  else {
    while(s->canReadLine()) {
      strBuffer += s->readLine();
    }
  }
}

void CWWWServer::slotDoClose()
{
    QApplication::restoreOverrideCursor();

  // close signal can occur when there is still data in read buffer
  if(s->bytesAvailable()) 
    slotDoRead();


  switch(state) {
  case ST_UPDATECATEGORY:
    doCategoryUpdate();
    break;
  case ST_UPDATEVULN:
    doVulnCategoryUpdate();
    break;
  case ST_FETCHABSTRACT:
    doAbstract();
    break;
  case ST_GETCONTENT:
    doContent();
    break;
  case ST_VULNPOLL:
    doVuln();
    break;
  case ST_FETCHAD:
    doAd();
    break;
  case ST_FETCHADIMAGE:
    doAdImage();
    break;
  case ST_IDLE:
  default:
    qWarning("Impossible state in CWWWServer::slotDoClose() !!\n");
  }
  s->close();
  goIdle();
}

void CWWWServer::doAbstract()
{
  emit newAbstract(strBuffer);
}
void CWWWServer::doContent()
{
  if(bDebug)
    cout << "content received" << endl;
    emit newContent(strBuffer);
}

void CWWWServer::doAd() 
{
  int begin, end;
  QString strGif;
  QString strLink;
  globalReplace(strBuffer, "&#58;", ":");
  globalReplace(strBuffer,"&amp;", "&");

  if((begin = strBuffer.find("<IMG SRC=\"", 0, true)) == -1)
    return;

  begin += 10;
  if((end = strBuffer.find("\"", begin, true)) == -1)
    return;
  
  strGif = strBuffer.mid(begin, end - begin);
  if((begin = strGif.find("/images", 0, true)) == -1)
    return;
 
  strGif = strGif.mid(begin);

  // Find link
  if((begin = strBuffer.find("<A HREF=\"", 0, true)) == -1)
    return;
  begin += 9;
  if((end = strBuffer.find("\"", begin, true)) == -1)
    return;

  strLink = strBuffer.mid(begin, end - begin);
  
  emit newAd(strGif, strLink);
}
void CWWWServer::doAdImage()
{
  adfile->close();
  delete adfile;
  emit newAdImage();
  return;
}
void CWWWServer::globalReplace(QString &strBuffer, QString strReplaceThis, QString strWithThis)
{
  int i;
  while((i = strBuffer.find(strReplaceThis, 0, false)) != -1) 
    strBuffer.replace(i, strReplaceThis.length(), strWithThis);
}
void CWWWServer::doVulnCategoryUpdate()
{
  QFile *f = new QFile(strPagerDir + "vulncategory.dat");


  if(f->open(IO_Truncate | IO_WriteOnly)) {
    f->writeBlock(strBuffer.ascii(), strBuffer.length());
    f->close();
    delete f;
    emit vulnCategoryUpdateDone();
    if(bDebug)
      cout << "Vuln categories updated" << endl;
  }
  else {
    delete f;
    QMessageBox::critical(0, "Error Updating Vulnerability Categories",
			  "Cannot open category file for writing", QMessageBox::Ok, 0);
  }
}
void CWWWServer::doVuln()
{
  if(bDebug)
    cout << "doVuln()" << endl;
  QString strTmp, strVuln;

  if(!xmlExtract("vulns", strBuffer, strTmp)) 
    return;

  if(strTmp.find("<!-- error old category file") != -1) {
    if(bDebug)
      cout << "Old vuln category file" << endl;
    queueRequest(CWWWRequest::R_UPDATEVULN, NULL);
    return;
  }
  if(strTmp.contains("<vuln>") == 0)
    return;

  QFile *f = new QFile(strPagerDir + "vulnpoll.dat");
  if(f->open(IO_Truncate | IO_WriteOnly)) {
    f->writeBlock(strTmp.ascii(), strTmp.length());
    f->close();
    delete f;
    emit newVulns();
  }
  else {
    delete f;
    QMessageBox::critical(0, "Error Saving Temporary", "Could not open temporary file from writing", QMessageBox::Ok, 0);
  }
}
void CWWWServer::doCategoryUpdate()
{
  QFile *f = new QFile(strPagerDir + "category.dat");
  QString strTmp;
  if(!xmlExtract("pre", strBuffer, strTmp)) {
    delete f;
    return;
  }

  if(f->exists()) {
    // make a copy of category file called category.old
    FileCopy("category.dat", "category.old");
  }

  if(f->open(IO_Truncate | IO_WriteOnly)) {
    f->writeBlock(strTmp.ascii(), strTmp.length());
    f->close();
    delete f;
    emit categoryUpdateDone();
  }
  else {
    delete f;
    QMessageBox::critical(0, "Error Updating Categories", 
			  "Cannot open category file for writing", 
			  QMessageBox::Ok, 0);
  }
}

bool CWWWServer::FileCopy(const QString &source, const QString &destination)
{
  QFile s(strPagerDir + source);
  QFile d(strPagerDir + destination);

  if(!s.open(IO_ReadOnly)) 
    return false;

  if(!d.open(IO_Truncate | IO_WriteOnly)) 
    return false;

  QTextStream in(&s);
  QTextStream out(&d);
  QString line;
  while(!in.eof()) {
    line = in.readLine();
    
    out << line << endl;
  }
  s.close();
  d.close();

  return true;
}


//
// Since we're only using this for abstracts, we'll use the very poor definition
// of an HTML tag being any pair of angle brackets with less than 200 characters
// in between.
//
void CWWWServer::removeTags(QString &str)
{
  int n = 0;
  int m;
  while( (n = str.find('<', n)) != -1) 
    if( (m = str.find('>', n + 1)) != -1) 
      if( (m - n - 1) < 200) 
	str.remove(n, m - n + 1);
      else
	n = m;
}
	
bool CWWWServer::xmlExtract(const QString &tag, QString &in, QString &out)
{
  QString strOpen, strClose;
  int openBegin, openEnd, closeBegin, closeEnd;
  char c;
  int i;

  strOpen.sprintf("<%s", tag.ascii());
  strClose.sprintf("</%s", tag.ascii());

  i = 0; c = 0;
  // If the character after strOpen is not a ' ' or a '>' keep searching
  while(c != ' ' && c != '>') {
    if( (openBegin = in.find(strOpen, i, false)) == -1)
      return false;
    c = in[openBegin + strOpen.length()].cell();
    i = openBegin + strOpen.length();
  }

  i = 0; c = 0;
  // If the character after strClose is not a ' ' or a '>' keep searching
  while(c != ' ' && c != '>') {
    if( (closeBegin = in.find(strClose, i, false)) == -1)
      return false;
    c = in[closeBegin + strClose.length()].cell();
    i = closeBegin + strClose.length();
  }

  if(closeBegin <= openBegin)
    return false;

  if( (openEnd = in.find('>', strOpen.length() + openBegin, false)) == -1)
    return false;


  if( (closeEnd = in.find('>', strClose.length() + closeBegin, false)) == -1)
    return false;
  


  // Extract text between tags
  out = in.mid(openEnd + 1, closeBegin - openEnd - 1);

  // Delete text and tags from original string
  
  in.remove(openBegin, closeEnd - openBegin + 1);

  return true;
}



  










